@@ -8,7 +8,7 @@ import os
 import shutil
 import sys
 from pathlib import Path
-from typing import Set, Union
+from typing import Set, Union, List, Tuple
 
 from rich.console import Console
 from rich.text import Text
@@ -16,7 +16,9 @@ from rich.theme import Theme
 from natsort import natsorted, ns
 import regex
 
-VERSION = "1.8.3"
+import se.easy_xml
+
+VERSION = "2.1.0"
 MESSAGE_INDENT = "    "
 UNICODE_BOM = "\ufeff"
 NO_BREAK_SPACE = "\u00a0"
@@ -26,7 +28,9 @@ ZERO_WIDTH_SPACE = "\ufeff"
 SHY_HYPHEN = "\u00ad"
 FUNCTION_APPLICATION = "\u2061"
 NO_BREAK_HYPHEN = "\u2011"
-IGNORED_FILENAMES = ["colophon.xhtml", "titlepage.xhtml", "imprint.xhtml", "uncopyright.xhtml", "halftitle.xhtml", "toc.xhtml", "loi.xhtml"]
+COMBINING_VERTICAL_LINE_ABOVE = "\u030d"
+COMBINING_ACUTE_ACCENT = "\u0301"
+INVISIBLE_TIMES = "\u2062"
 SELECTORS_TO_SIMPLIFY = [":first-child", ":only-child", ":last-child", ":nth-child", ":nth-last-child", ":first-of-type", ":only-of-type", ":last-of-type", ":nth-of-type", ":nth-last-of-type"]
 MESSAGE_TYPE_WARNING = 1
 MESSAGE_TYPE_ERROR = 2
@@ -114,6 +118,10 @@ class BuildFailedException(SeException):
 	""" Build failed """
 	code = 17
 
+	def __init__(self, message, messages: List = None):
+		super().__init__(message)
+		self.messages = messages if messages else []
+
 def strip_bom(string: str) -> str:
 	"""
 	Remove the Unicode Byte Order Mark from a string.
@@ -130,17 +138,19 @@ def strip_bom(string: str) -> str:
 
 	return string
 
-def quiet_remove(file: Path) -> None:
+def prep_output(message: str, plain_output: bool = False) -> str:
 	"""
-	Helper function to delete a file without throwing an exception if the file doesn't exist.
+	Return a message formatted for the chosen output style, i.e., color or plain.
 	"""
 
-	try:
-		file.unlink()
-	except Exception:
-		pass
+	if plain_output:
+		# Replace color markup with `
+		message = regex.sub(r"\[(?:/|xhtml|xml|val|attr|css|val|class|path|url|text|bash|link)(?:=[^\]]*?)*\]", "`", message)
+		message = regex.sub(r"`+", "`", message)
 
-def print_error(message: Union[SeException, str], verbose: bool = False, is_warning: bool = False) -> None:
+	return message
+
+def print_error(message: Union[SeException, str], verbose: bool = False, is_warning: bool = False, plain_output: bool = False) -> None:
 	"""
 	Helper function to print a colored error message to the console.
 
@@ -158,7 +168,7 @@ def print_error(message: Union[SeException, str], verbose: bool = False, is_warn
 	"""
 
 	label = "Error" if not is_warning else "Warning"
-	bg_color = 'red' if not is_warning else 'yellow'
+	bg_color = "red" if not is_warning else "yellow"
 
 	# We have to print to stdout in case we're called from GNU Parallel, otherwise weird newline issues occur
 	# This no longer works with rich because it can't (yet) output to stderr
@@ -170,7 +180,13 @@ def print_error(message: Union[SeException, str], verbose: bool = False, is_warn
 		message = str(message).replace("\n", f"\n{MESSAGE_INDENT}")
 
 	console = Console(file=output_file, highlight=False, theme=RICH_THEME, force_terminal=is_called_from_parallel()) # Syntax highlighting will do weird things when printing paths; force_terminal prints colors when called from GNU Parallel
-	console.print(f"{MESSAGE_INDENT if verbose else ''}[white on {bg_color} bold] {label} [/] {message}")
+
+	if plain_output:
+		# Replace color markup with `
+		message = prep_output(message, True)
+		console.print(f"{MESSAGE_INDENT if verbose else ''}[{label}] {message}")
+	else:
+		console.print(f"{MESSAGE_INDENT if verbose else ''}[white on {bg_color} bold] {label} [/] {message}")
 
 def is_positive_integer(value: str) -> int:
 	"""
@@ -178,47 +194,59 @@ def is_positive_integer(value: str) -> int:
 	Raise an exception if value is not a positive integer.
 	"""
 
-	int_value = int(value)
-	if int_value <= 0:
-		raise argparse.ArgumentTypeError(f"{value} is not a positive integer")
+	try:
+		int_value = int(value)
+		if int_value <= 0:
+			raise argparse.ArgumentTypeError(f"{value} is not a positive integer")
+	except Exception as ex:
+		raise argparse.ArgumentTypeError(f"{value} is not a positive integer") from ex
 
 	return int_value
 
-def get_target_filenames(targets: list, allowed_extensions: tuple, ignored_filenames: list = None) -> list:
+def quiet_remove(file: Path) -> None:
+	"""
+	Helper function to delete a file without crashing if it doesn't exist.
+	This has to remain until the SE server is on Ubuntu 22.04 or Python 3.8.5+
+	"""
+
+	try:
+		file.unlink()
+	except Exception:
+		pass
+
+def get_target_filenames(targets: list, allowed_extensions: Union[tuple, str]) -> list:
 	"""
 	Helper function to convert a list of filenames or directories into a list of filenames based on some parameters.
 
+	allowed_extensions is only applied on targets that are directories.
+
 	INPUTS
 	targets: A list of filenames or directories
 	allowed_extensions: A tuple containing a series of allowed filename extensions; extensions must begin with "."
-	ignored_filenames: If None, ignore files in the se.IGNORED_FILENAMES constant. If a list, ignore that list of filenames.
-				Pass an empty list to ignore no files.
 
 	OUTPUTS
 	A set of file paths and filenames contained in the target list.
 	"""
 
-	if ignored_filenames is None:
-		ignored_filenames = IGNORED_FILENAMES
-
 	target_xhtml_filenames = set()
 
+	if isinstance(allowed_extensions, str):
+		allowed_extensions = (allowed_extensions,)
+
 	for target in targets:
 		target = Path(target).resolve()
 
 		if target.is_dir():
-			for root, _, filenames in os.walk(target):
-				for filename in filenames:
-					if allowed_extensions:
-						if filename.endswith(allowed_extensions):
-							if filename not in ignored_filenames:
-								target_xhtml_filenames.add(Path(root) / filename)
-					else:
-						if filename not in ignored_filenames:
-							target_xhtml_filenames.add(Path(root) / filename)
+			for file_path in target.glob("**/*"):
+				file_path.resolve()
+				if allowed_extensions:
+					if file_path.suffix in allowed_extensions:
+						target_xhtml_filenames.add(file_path)
+				else:
+					target_xhtml_filenames.add(file_path)
 		else:
-			if target.name.endswith(allowed_extensions):
-				target_xhtml_filenames.add(target)
+			# If we're looking at an actual file, just add it regardless of whether it's ignored
+			target_xhtml_filenames.add(target)
 
 	return natsorted(list(target_xhtml_filenames), key=lambda x: str(x.name), alg=ns.PATH)
 
@@ -244,3 +272,45 @@ def is_called_from_parallel(return_none=True) -> Union[bool,None]:
 		pass
 
 	return None if return_none else False
+
+def get_dom_if_not_ignored(xhtml: str, ignored_types: List[str] = None) -> Tuple[bool, Union[None, se.easy_xml.EasyXmlTree]]:
+	"""
+	Given a string of XHTML, return a dom tree ONLY IF the dom does not contain a
+	top-level <section> element with any of the passed semantics.
+
+	Pass an empty list to ignored_types to ignore nothing.
+	Pass None to ignored_types to ignore a default set of SE files.
+
+	RETURNS
+	A tuple of (is_ignored, dom)
+	If the file is ignored, is_ignored will be True.
+	If the dom couldn't be created (for example it is invalid XML) then the dom part
+	of the tuple will be None.
+	"""
+
+	is_ignored = False
+	ignored_regex = None
+	dom = None
+
+	try:
+		dom = se.easy_xml.EasyXmlTree(xhtml)
+	except:
+		return (False, None)
+
+	# Ignore some SE files
+	# Default ignore list
+	if ignored_types is None:
+		ignored_regex = "(colophon|titlepage|imprint|copyright-page|halftitlepage|toc|loi)"
+
+	elif len(ignored_types) > 0:
+		ignored_regex = "("
+		for item in ignored_types:
+			ignored_regex = f"{ignored_regex}{regex.escape(item)}|"
+
+		ignored_regex = ignored_regex.rstrip("|") + ")"
+
+	if ignored_regex:
+		if dom.xpath(f"/html[re:test(@epub:prefix, '[\\s\\b]se:[\\s\\b]')]/body/*[(name() = 'section' or name() = 'nav') and re:test(@epub:type, '\\b{ignored_regex}\\b')]"):
+			is_ignored = True
+
+	return (is_ignored, dom)
