@@ -5,7 +5,6 @@ This module implements the `se create_draft` command.
 import argparse
 from io import StringIO
 import shutil
-import unicodedata
 import urllib.parse
 from argparse import Namespace
 from pathlib import Path
@@ -18,6 +17,7 @@ import requests
 from rich.console import Console
 from ftfy import fix_text
 from lxml import etree
+from unidecode import unidecode
 
 import se
 import se.formatting
@@ -55,7 +55,7 @@ CONTRIBUTOR_BLOCK_TEMPLATE = """<dc:contributor id="CONTRIBUTOR_ID">CONTRIBUTOR_
 		<meta property="file-as" refines="#CONTRIBUTOR_ID">CONTRIBUTOR_SORT</meta>
 		<meta property="se:name.person.full-name" refines="#CONTRIBUTOR_ID">CONTRIBUTOR_FULL_NAME</meta>
 		<meta property="se:url.encyclopedia.wikipedia" refines="#CONTRIBUTOR_ID">CONTRIBUTOR_WIKI_URL</meta>
-		<meta property="se:url.authority.nacoaf" refines="#CONTRIBUTOR_ID">CONTRIBUTOR_NACOAF_URL</meta>
+		<meta property="se:url.authority.nacoaf" refines="#CONTRIBUTOR_ID">CONTRIBUTOR_NACOAF_URI</meta>
 		<meta property="role" refines="#CONTRIBUTOR_ID" scheme="marc:relators">CONTRIBUTOR_MARC</meta>"""
 
 def _replace_in_file(file_path: Path, search: Union[str, list], replace: Union[str, list]) -> None:
@@ -98,7 +98,12 @@ def _get_word_widths(string: str, target_height: int) -> list:
 
 		for char in word:
 			# Convert accented characters to unaccented characters
-			char = regex.sub(r"\p{M}", "", unicodedata.normalize("NFKD", char))
+			char = unidecode(char)
+
+			# Oops! unidecode also converts `’` to `'`. Change it back here
+			if char == "'":
+				char = "’"
+
 			width += int(LEAGUE_SPARTAN_100_WIDTHS[char] * target_height / 100) + LEAGUE_SPARTAN_KERNING + LEAGUE_SPARTAN_AVERAGE_SPACING
 
 		width = width - LEAGUE_SPARTAN_KERNING - LEAGUE_SPARTAN_AVERAGE_SPACING
@@ -121,7 +126,6 @@ def _calculate_image_lines(string: str, target_height: int, canvas_width: int) -
 	OUTPUTS
 	An array of strings. Each string represents one line of text in the final image. The lines are ordered with the widest at the bottom.
 	"""
-
 	words = _get_word_widths(string, target_height)
 	lines = []
 	current_line = ""
@@ -347,16 +351,16 @@ def _generate_cover_svg(title: str, authors: List[str], title_string: str) -> st
 
 	return svg
 
-def _get_wikipedia_url(string: str, get_nacoaf_url: bool) -> Tuple[Optional[str], Optional[str]]:
+def _get_wikipedia_url(string: str, get_nacoaf_uri: bool) -> Tuple[Optional[str], Optional[str]]:
 	"""
 	Given a string, try to see if there's a Wikipedia page entry, and an optional NACOAF entry, for that string.
 
 	INPUTS
 	string: The string to find on Wikipedia
-	get_nacoaf_url: Include NACOAF URL in resulting tuple, if found?
+	get_nacoaf_uri: Include NACOAF URI in resulting tuple, if found?
 
 	OUTPUTS
-	A tuple of two strings. The first string is the Wikipedia URL, the second is the NACOAF URL.
+	A tuple of two strings. The first string is the Wikipedia URL, the second is the NACOAF URI.
 	"""
 
 	# We try to get the Wikipedia URL by the subject by taking advantage of the fact that Wikipedia's special search will redirect you immediately
@@ -366,25 +370,25 @@ def _get_wikipedia_url(string: str, get_nacoaf_url: bool) -> Tuple[Optional[str]
 	try:
 		response = requests.get("https://en.wikipedia.org/wiki/Special:Search", params={"search": string, "go": "Go", "ns0": "1"}, allow_redirects=False)
 	except Exception as ex:
-		se.print_error(f"Couldn’t contact Wikipedia. Exception: {ex}")
+		raise se.RemoteCommandErrorException(f"Couldn’t contact Wikipedia. Exception: {ex}") from ex
 
 	if response.status_code == 302:
-		nacoaf_url = None
+		nacoaf_uri = None
 		wiki_url = response.headers["Location"]
 		if urllib.parse.urlparse(wiki_url).path == "/wiki/Special:Search":
 			# Redirected back to search URL, no match
 			return None, None
 
-		if get_nacoaf_url:
+		if get_nacoaf_uri:
 			try:
 				response = requests.get(wiki_url)
 			except Exception as ex:
-				se.print_error(f"Couldn’t contact Wikipedia. Exception: {ex}")
+				raise se.RemoteCommandErrorException(f"Couldn’t contact Wikipedia. Exception: {ex}") from ex
 
 			for match in regex.findall(r"https?://id\.loc\.gov/authorities/names/n[0-9]+", response.text):
-				nacoaf_url = match
+				nacoaf_uri = match.replace("https:","http:")
 
-		return wiki_url, nacoaf_url
+		return wiki_url, nacoaf_uri
 
 	return None, None
 
@@ -395,10 +399,19 @@ def _copy_template_file(filename: str, dest_path: Path) -> None:
 	with importlib_resources.path("se.data.templates", filename) as src_path:
 		shutil.copy(src_path, dest_path)
 
+def _add_name_abbr(contributor: str) -> str:
+	"""
+	Add <abbr epub:type="z3998:given-name"> around contributor names
+	"""
+
+	contributor = regex.sub(r"([\p{Uppercase_Letter}]\.(?:\s*[\p{Uppercase_Letter}]\.)*)", r"""<abbr epub:type="z3998:given-name">\1</abbr>""", contributor)
+
+	return contributor
+
 def _generate_contributor_string(contributors: List[Dict], include_xhtml: bool) -> str:
 	"""
 	Given a list of contributors, generate a contributor string like `Bob Smith, Jane Doe, and Sam Johnson`.
-	With include_xhtml, the string looks like: `<b class="name">Bob Smith</b>, <a href="https://en.wikipedia.org/wiki/Jane_Doe">Jane Doe</a>, and <b class="name">Sam Johnson</b>`
+	With include_xhtml, the string looks like: `<b epub:type="z3998:personal-name">Bob Smith</b>, <a href="https://en.wikipedia.org/wiki/Jane_Doe">Jane Doe</a>, and <b epub:type="z3998:personal-name">Sam Johnson</b>`
 
 	INPUTS
 	contributors: A list of contributor dicts
@@ -416,25 +429,25 @@ def _generate_contributor_string(contributors: List[Dict], include_xhtml: bool)
 	if len(contributors) == 1:
 		if include_xhtml:
 			if contributors[0]["wiki_url"]:
-				output += f"<a href=\"{contributors[0]['wiki_url']}\">{contributors[0]['name']}</a>"
+				output += f"""<a href="{contributors[0]['wiki_url']}">{_add_name_abbr(contributors[0]['name'])}</a>"""
 			else:
-				output += f"<b class=\"name\">{contributors[0]['name']}</b>"
+				output += f"""<b epub:type="z3998:personal-name">{_add_name_abbr(contributors[0]['name'])}</b>"""
 		else:
 			output += contributors[0]["name"]
 
 	elif len(contributors) == 2:
 		if include_xhtml:
 			if contributors[0]["wiki_url"]:
-				output += f"<a href=\"{contributors[0]['wiki_url']}\">{contributors[0]['name']}</a>"
+				output += f"""<a href="{contributors[0]['wiki_url']}">{_add_name_abbr(contributors[0]['name'])}</a>"""
 			else:
-				output += f"<b class=\"name\">{contributors[0]['name']}</b>"
+				output += f"""<b epub:type="z3998:personal-name">{_add_name_abbr(contributors[0]['name'])}</b>"""
 
 			output += " and "
 
 			if contributors[1]["wiki_url"]:
-				output += f"<a href=\"{contributors[1]['wiki_url']}\">{contributors[1]['name']}</a>"
+				output += f"""<a href="{contributors[1]['wiki_url']}">{_add_name_abbr(contributors[1]['name'])}</a>"""
 			else:
-				output += f"<b class=\"name\">{contributors[1]['name']}</b>"
+				output += f"""<b epub:type="z3998:personal-name">{_add_name_abbr(contributors[1]['name'])}</b>"""
 		else:
 			output += contributors[0]["name"] + " and " + contributors[1]["name"]
 
@@ -448,9 +461,9 @@ def _generate_contributor_string(contributors: List[Dict], include_xhtml: bool)
 
 			if include_xhtml:
 				if contributor["wiki_url"]:
-					output += f"<a href=\"{contributor['wiki_url']}\">{contributor['name']}</a>"
+					output += f"""<a href="{contributor['wiki_url']}">{_add_name_abbr(contributor['name'])}</a>"""
 				else:
-					output += f"<b class=\"name\">{contributor['name']}</b>"
+					output += f"""<b epub:type="z3998:personal-name">{_add_name_abbr(contributor['name'])}</b>"""
 			else:
 				output += contributor["name"]
 
@@ -476,8 +489,8 @@ def _generate_metadata_contributor_xml(contributors: List[Dict], contributor_typ
 		if contributor["wiki_url"]:
 			contributor_block = contributor_block.replace(">CONTRIBUTOR_WIKI_URL<", f">{contributor['wiki_url']}<")
 
-		if contributor["nacoaf_url"]:
-			contributor_block = contributor_block.replace(">CONTRIBUTOR_NACOAF_URL<", f">{contributor['nacoaf_url']}<")
+		if contributor["nacoaf_uri"]:
+			contributor_block = contributor_block.replace(">CONTRIBUTOR_NACOAF_URI<", f">{contributor['nacoaf_uri']}<")
 
 		# Make an attempt at figuring out the file-as name. We check for some common two-word last names.
 		matches = regex.findall(r"^(.+?)\s+((?:(?:Da|Das|De|Del|Della|Di|Du|El|La|Le|Van|Van Der|Von)\s+)?[^\s]+)$", contributor["name"])
@@ -488,7 +501,7 @@ def _generate_metadata_contributor_xml(contributors: List[Dict], contributor_typ
 			contributor_block = contributor_block.replace(">CONTRIBUTOR_SORT<", ">Anonymous<")
 			contributor_block = contributor_block.replace("""<meta property="se:name.person.full-name" refines="#ID">CONTRIBUTOR_FULL_NAME</meta>""", "")
 			contributor_block = contributor_block.replace("""<meta property="se:url.encyclopedia.wikipedia" refines="#ID">CONTRIBUTOR_WIKI_URL</meta>""", "")
-			contributor_block = contributor_block.replace("""<meta property="se:url.authority.nacoaf" refines="#ID">CONTRIBUTOR_NACOAF_URL</meta>""", "")
+			contributor_block = contributor_block.replace("""<meta property="se:url.authority.nacoaf" refines="#ID">CONTRIBUTOR_NACOAF_URI</meta>""", "")
 
 		contributor_block = contributor_block.replace(">CONTRIBUTOR_NAME<", f">{contributor['name']}<")
 		contributor_block = contributor_block.replace("id=\"CONTRIBUTOR_ID\"", f"id=\"{contributor_type}-{i + 1}\"")
@@ -520,18 +533,18 @@ def _create_draft(args: Namespace):
 	translators = []
 	illustrators = []
 	pg_producers = []
-	title = args.title.replace("'", "’")
+	title = se.formatting.titlecase(args.title.replace("'", "’"))
 
 	for author in args.author:
-		authors.append({"name": author.replace("'", "’"), "wiki_url": None, "nacoaf_url": None})
+		authors.append({"name": author.replace("'", "’"), "wiki_url": None, "nacoaf_uri": None})
 
 	if args.translator:
 		for translator in args.translator:
-			translators.append({"name": translator.replace("'", "’"), "wiki_url": None, "nacoaf_url": None})
+			translators.append({"name": translator.replace("'", "’"), "wiki_url": None, "nacoaf_uri": None})
 
 	if args.illustrator:
 		for illustrator in args.illustrator:
-			illustrators.append({"name": illustrator.replace("'", "’"), "wiki_url": None, "nacoaf_url": None})
+			illustrators.append({"name": illustrator.replace("'", "’"), "wiki_url": None, "nacoaf_uri": None})
 
 	title_string = title
 	if authors and authors[0]["name"].lower() != "anonymous":
@@ -572,31 +585,37 @@ def _create_draft(args: Namespace):
 	if repo_path.is_dir():
 		raise se.InvalidInputException(f"Directory already exists: [path][link=file://{repo_path}]{repo_path}[/][/].")
 
-	# Get data on authors
-	for i, author in enumerate(authors):
-		if not args.offline and author["name"].lower() != "anonymous":
-			author["wiki_url"], author["nacoaf_url"] = _get_wikipedia_url(author["name"], True)
+	content_path = repo_path / "src"
 
-	# Get data on translators
-	for i, translator in enumerate(translators):
-		if not args.offline and translator["name"].lower() != "anonymous":
-			translator["wiki_url"], translator["nacoaf_url"] = _get_wikipedia_url(translator["name"], True)
+	if args.white_label:
+		content_path = repo_path
 
-	# Get data on illlustrators
-	for i, illustrator in enumerate(illustrators):
-		if not args.offline and illustrator["name"].lower() != "anonymous":
-			illustrator["wiki_url"], illustrator["nacoaf_url"] = _get_wikipedia_url(illustrator["name"], True)
+	if not args.white_label:
+		# Get data on authors
+		for i, author in enumerate(authors):
+			if not args.offline and author["name"].lower() != "anonymous":
+				author["wiki_url"], author["nacoaf_uri"] = _get_wikipedia_url(author["name"], True)
+
+		# Get data on translators
+		for i, translator in enumerate(translators):
+			if not args.offline and translator["name"].lower() != "anonymous":
+				translator["wiki_url"], translator["nacoaf_uri"] = _get_wikipedia_url(translator["name"], True)
+
+		# Get data on illlustrators
+		for i, illustrator in enumerate(illustrators):
+			if not args.offline and illustrator["name"].lower() != "anonymous":
+				illustrator["wiki_url"], illustrator["nacoaf_uri"] = _get_wikipedia_url(illustrator["name"], True)
 
 	# Download PG HTML and do some fixups
-	if args.pg_url:
+	if args.pg_id:
 		if args.offline:
 			raise se.RemoteCommandErrorException("Cannot download Project Gutenberg ebook when offline option is enabled.")
 
-		args.pg_url = args.pg_url.replace("http://", "https://")
+		pg_url = f"https://www.gutenberg.org/ebooks/{args.pg_id}"
 
 		# Get the ebook metadata
 		try:
-			response = requests.get(args.pg_url)
+			response = requests.get(pg_url)
 			pg_metadata_html = response.text
 		except Exception as ex:
 			raise se.RemoteCommandErrorException(f"Couldn’t download Project Gutenberg ebook metadata page. Exception: {ex}")
@@ -644,16 +663,18 @@ def _create_draft(args: Namespace):
 			pg_language = "en-GB"
 
 	# Create necessary directories
-	(repo_path / "images").mkdir(parents=True)
-	(repo_path / "src" / "epub" / "css").mkdir(parents=True)
-	(repo_path / "src" / "epub" / "images").mkdir(parents=True)
-	(repo_path / "src" / "epub" / "text").mkdir(parents=True)
-	(repo_path / "src" / "META-INF").mkdir(parents=True)
+	(content_path / "epub" / "css").mkdir(parents=True)
+	(content_path / "epub" / "images").mkdir(parents=True)
+	(content_path / "epub" / "text").mkdir(parents=True)
+	(content_path / "META-INF").mkdir(parents=True)
+
+	if not args.white_label:
+		(repo_path / "images").mkdir(parents=True)
 
 	is_pg_html_parsed = True
 
 	# Write PG data if we have it
-	if args.pg_url and pg_ebook_html:
+	if args.pg_id and pg_ebook_html:
 		try:
 			dom = etree.parse(StringIO(regex.sub(r"encoding=\".+?\"", "", pg_ebook_html)), parser)
 			namespaces = {"re": "http://exslt.org/regular-expressions"}
@@ -699,7 +720,7 @@ def _create_draft(args: Namespace):
 			# lxml can also output duplicate default namespace declarations so remove the first one only
 			output = regex.sub(r"(xmlns=\".+?\")(\sxmlns=\".+?\")+", r"\1", output)
 
-			with open(repo_path / "src" / "epub" / "text" / "body.xhtml", "w", encoding="utf-8") as file:
+			with open(content_path / "epub" / "text" / "body.xhtml", "w", encoding="utf-8") as file:
 				file.write(output)
 
 		except OSError as ex:
@@ -708,113 +729,120 @@ def _create_draft(args: Namespace):
 			# Save this error for later, because it's still useful to complete the create-draft process
 			# even if we've failed to parse PG's HTML source.
 			is_pg_html_parsed = False
-			se.quiet_remove(repo_path / "src" / "epub" / "text" / "body.xhtml")
+			se.quiet_remove(content_path / "epub" / "text" / "body.xhtml")
 
 	# Copy over templates
 	_copy_template_file("gitignore", repo_path / ".gitignore")
-	_copy_template_file("LICENSE.md", repo_path)
-	_copy_template_file("container.xml", repo_path / "src" / "META-INF")
-	_copy_template_file("mimetype", repo_path / "src")
-	_copy_template_file("content.opf", repo_path / "src" / "epub")
-	_copy_template_file("onix.xml", repo_path / "src" / "epub")
-	_copy_template_file("toc.xhtml", repo_path / "src" / "epub")
-	_copy_template_file("core.css", repo_path / "src" / "epub" / "css")
-	_copy_template_file("local.css", repo_path / "src" / "epub" / "css")
-	_copy_template_file("se.css", repo_path / "src" / "epub" / "css")
-	_copy_template_file("logo.svg", repo_path / "src" / "epub" / "images")
-	_copy_template_file("colophon.xhtml", repo_path / "src" / "epub" / "text")
-	_copy_template_file("imprint.xhtml", repo_path / "src" / "epub" / "text")
-	_copy_template_file("titlepage.xhtml", repo_path / "src" / "epub" / "text")
-	_copy_template_file("uncopyright.xhtml", repo_path / "src" / "epub" / "text")
-	_copy_template_file("titlepage.svg", repo_path / "images")
-	_copy_template_file("cover.jpg", repo_path / "images" / "cover.jpg")
-	_copy_template_file("cover.svg", repo_path / "images" / "cover.svg")
+	_copy_template_file("container.xml", content_path / "META-INF")
+	_copy_template_file("mimetype", content_path)
+	_copy_template_file("onix.xml", content_path / "epub")
+	_copy_template_file("toc.xhtml", content_path / "epub")
+	_copy_template_file("core.css", content_path / "epub" / "css")
+	_copy_template_file("local.css", content_path / "epub" / "css")
+	_copy_template_file("cover.jpg", repo_path / "images")
+
+	if args.white_label:
+		_copy_template_file("content-white-label.opf", content_path / "epub" / "content.opf")
+		_copy_template_file("titlepage-white-label.xhtml", content_path / "epub" / "text" / "titlepage.xhtml")
+		_copy_template_file("cover.jpg", content_path / "epub" / "images")
+
+	else:
+		_copy_template_file("cover.svg", repo_path / "images")
+		_copy_template_file("titlepage.svg", repo_path / "images")
+		_copy_template_file("se.css", content_path / "epub" / "css")
+		_copy_template_file("logo.svg", content_path / "epub" / "images")
+		_copy_template_file("colophon.xhtml", content_path / "epub" / "text")
+		_copy_template_file("imprint.xhtml", content_path / "epub" / "text")
+		_copy_template_file("uncopyright.xhtml", content_path / "epub" / "text")
+		_copy_template_file("titlepage.xhtml", content_path / "epub" / "text")
+		_copy_template_file("content.opf", content_path / "epub")
+		_copy_template_file("LICENSE.md", repo_path)
 
 	# Try to find Wikipedia links if possible
 	ebook_wiki_url = None
 
-	if not args.offline and title != "Short Fiction":
-		# There's a "Short Fiction" Wikipedia article, so make an exception for that case
+	if not args.offline and title not in ("Short Fiction", "Poetry", "Essays", "Plays"):
 		ebook_wiki_url, _ = _get_wikipedia_url(title, False)
 
-	# Pre-fill a few templates
-	_replace_in_file(repo_path / "src" / "epub" / "text" / "titlepage.xhtml", "TITLE_STRING", title_string)
-	_replace_in_file(repo_path / "images" / "titlepage.svg", "TITLE_STRING", title_string)
-	_replace_in_file(repo_path / "images" / "cover.svg", "TITLE_STRING", title_string)
+	if args.white_label:
+		_replace_in_file(content_path / "epub" / "text" / "titlepage.xhtml", "TITLE", title)
 
-	# Create the titlepage SVG
-	contributors = {}
-	if args.translator:
-		contributors["translated by"] = _generate_contributor_string(translators, False)
-
-	if args.illustrator:
-		contributors["illustrated by"] = _generate_contributor_string(illustrators, False)
-
-	with open(repo_path / "images" / "titlepage.svg", "w", encoding="utf-8") as file:
-		file.write(_generate_titlepage_svg(title, [author["name"] for author in authors], contributors, title_string))
-
-	# Create the cover SVG
-	with open(repo_path / "images" / "cover.svg", "w", encoding="utf-8") as file:
-		file.write(_generate_cover_svg(title, [author["name"] for author in authors], title_string))
-
-	# Build the cover/titlepage for distribution
-	epub = SeEpub(repo_path)
-	epub.generate_cover_svg()
-	epub.generate_titlepage_svg()
-
-	if args.pg_url:
-		_replace_in_file(repo_path / "src" / "epub" / "text" / "imprint.xhtml", "PG_URL", args.pg_url)
-
-	# Fill out the colophon
-	with open(repo_path / "src" / "epub" / "text" / "colophon.xhtml", "r+", encoding="utf-8") as file:
-		colophon_xhtml = file.read()
-
-		colophon_xhtml = colophon_xhtml.replace("SE_IDENTIFIER", identifier)
-		colophon_xhtml = colophon_xhtml.replace("TITLE", title)
-
-		contributor_string =  _generate_contributor_string(authors, True)
-
-		if contributor_string == "":
-			colophon_xhtml = colophon_xhtml.replace(" by<br/>\n\t\t\t<a href=\"AUTHOR_WIKI_URL\">AUTHOR</a>", contributor_string)
-		else:
-			colophon_xhtml = colophon_xhtml.replace("<a href=\"AUTHOR_WIKI_URL\">AUTHOR</a>", contributor_string)
+	else:
+		_replace_in_file(content_path / "epub" / "text" / "titlepage.xhtml", "TITLE_STRING", title_string)
 
+		# Create the titlepage SVG
+		contributors = {}
 		if translators:
-			translator_block = f"It was translated from ORIGINAL_LANGUAGE in TRANSLATION_YEAR by<br/>\n\t\t\t{_generate_contributor_string(translators, True)}.</p>"
-			colophon_xhtml = colophon_xhtml.replace("</p>\n\t\t\t<p>This ebook was produced for the<br/>", f"<br/>\n\t\t\t{translator_block}\n\t\t\t<p>This ebook was produced for the<br/>")
+			contributors["translated by"] = _generate_contributor_string(translators, False)
 
-		if args.pg_url:
-			colophon_xhtml = colophon_xhtml.replace("PG_URL", args.pg_url)
+		if illustrators:
+			contributors["illustrated by"] = _generate_contributor_string(illustrators, False)
 
-			if pg_publication_year:
-				colophon_xhtml = colophon_xhtml.replace("PG_YEAR", pg_publication_year)
+		with open(repo_path / "images" / "titlepage.svg", "w", encoding="utf-8") as file:
+			file.write(_generate_titlepage_svg(title, [author["name"] for author in authors], contributors, title_string))
 
-			if pg_producers:
-				producers_xhtml = ""
-				for i, producer in enumerate(pg_producers):
-					if "Distributed Proofread" in producer:
-						producers_xhtml = producers_xhtml + "<a href=\"https://www.pgdp.net\">The Online Distributed Proofreading Team</a>"
-					elif "anonymous" in producer.lower():
-						producers_xhtml = producers_xhtml + "<b class=\"name\">An Anonymous Volunteer</b>"
-					else:
-						producers_xhtml = producers_xhtml + f"<b class=\"name\">{producer.strip('.')}</b>"
+		# Create the cover SVG
+		with open(repo_path / "images" / "cover.svg", "w", encoding="utf-8") as file:
+			file.write(_generate_cover_svg(title, [author["name"] for author in authors], title_string))
 
-					if i < len(pg_producers) - 1:
-						producers_xhtml = producers_xhtml + ", "
+		# Build the cover/titlepage for distribution
+		epub = SeEpub(repo_path)
+		epub.generate_cover_svg()
+		epub.generate_titlepage_svg()
 
-					if i == len(pg_producers) - 2:
-						producers_xhtml = producers_xhtml + "and "
+		if args.pg_id:
+			_replace_in_file(content_path / "epub" / "text" / "imprint.xhtml", "PG_URL", pg_url)
 
-				producers_xhtml = producers_xhtml + "<br/>"
+		# Fill out the colophon
+		with open(content_path / "epub" / "text" / "colophon.xhtml", "r+", encoding="utf-8") as file:
+			colophon_xhtml = file.read()
 
-				colophon_xhtml = colophon_xhtml.replace("<b class=\"name\">TRANSCRIBER_1</b>, <b class=\"name\">TRANSCRIBER_2</b>, and <a href=\"https://www.pgdp.net\">The Online Distributed Proofreading Team</a><br/>", producers_xhtml)
+			colophon_xhtml = colophon_xhtml.replace("SE_IDENTIFIER", identifier)
+			colophon_xhtml = colophon_xhtml.replace("TITLE", title)
 
-		file.seek(0)
-		file.write(colophon_xhtml)
-		file.truncate()
+			contributor_string =  _generate_contributor_string(authors, True)
+
+			if contributor_string == "":
+				colophon_xhtml = colophon_xhtml.replace(" by<br/>\n\t\t\t<a href=\"AUTHOR_WIKI_URL\">AUTHOR</a>", contributor_string)
+			else:
+				colophon_xhtml = colophon_xhtml.replace("<a href=\"AUTHOR_WIKI_URL\">AUTHOR</a>", contributor_string)
+
+			if translators:
+				translator_block = f"It was translated from ORIGINAL_LANGUAGE in TRANSLATION_YEAR by<br/>\n\t\t\t{_generate_contributor_string(translators, True)}.</p>"
+				colophon_xhtml = colophon_xhtml.replace("</p>\n\t\t\t<p>This ebook was produced for the<br/>", f"<br/>\n\t\t\t{translator_block}\n\t\t\t<p>This ebook was produced for the<br/>")
+
+			if args.pg_id:
+				colophon_xhtml = colophon_xhtml.replace("PG_URL", pg_url)
+
+				if pg_publication_year:
+					colophon_xhtml = colophon_xhtml.replace("PG_YEAR", pg_publication_year)
+
+				if pg_producers:
+					producers_xhtml = ""
+					for i, producer in enumerate(pg_producers):
+						if "Distributed Proofread" in producer:
+							producers_xhtml = producers_xhtml + """<a href="https://www.pgdp.net">The Online Distributed Proofreading Team</a>"""
+						elif "anonymous" in producer.lower():
+							producers_xhtml = producers_xhtml + """<b epub:type="z3998:personal-name">An Anonymous Volunteer</b>"""
+						else:
+							producers_xhtml = producers_xhtml + f"""<b epub:type="z3998:personal-name">{_add_name_abbr(producer).strip('.')}</b>"""
+
+						if i < len(pg_producers) - 1:
+							producers_xhtml = producers_xhtml + ", "
+
+						if i == len(pg_producers) - 2:
+							producers_xhtml = producers_xhtml + "and "
+
+					producers_xhtml = producers_xhtml + "<br/>"
+
+					colophon_xhtml = colophon_xhtml.replace("""<b epub:type="z3998:personal-name">TRANSCRIBER_1</b>, <b epub:type="z3998:personal-name">TRANSCRIBER_2</b>, and <a href=\"https://www.pgdp.net\">The Online Distributed Proofreading Team</a><br/>""", producers_xhtml)
+
+			file.seek(0)
+			file.write(colophon_xhtml)
+			file.truncate()
 
 	# Fill out the metadata file
-	with open(repo_path / "src" / "epub" / "content.opf", "r+", encoding="utf-8") as file:
+	with open(content_path / "epub" / "content.opf", "r+", encoding="utf-8") as file:
 		metadata_xml = file.read()
 
 		metadata_xml = metadata_xml.replace("SE_IDENTIFIER", identifier)
@@ -826,13 +854,23 @@ def _create_draft(args: Namespace):
 			producers_xhtml = ""
 			i = 1
 			for producer in pg_producers:
+				# Name and File-As
 				if "Distributed Proofread" in producer:
-					producers_xhtml = producers_xhtml + f"\t\t<dc:contributor id=\"transcriber-{i}\">The Online Distributed Proofreading Team</dc:contributor>\n\t\t<meta property=\"file-as\" refines=\"#transcriber-{i}\">Online Distributed Proofreading Team, The</meta>\n\t\t<meta property=\"se:url.homepage\" refines=\"#transcriber-{i}\">https://pgdp.net</meta>\n"
+					producers_xhtml = producers_xhtml + f"\t\t<dc:contributor id=\"transcriber-{i}\">The Online Distributed Proofreading Team</dc:contributor>\n\t\t<meta property=\"file-as\" refines=\"#transcriber-{i}\">Online Distributed Proofreading Team, The</meta>\n"
 				elif "anonymous" in producer.lower():
 					producers_xhtml = producers_xhtml + f"\t\t<dc:contributor id=\"transcriber-{i}\">An Anonymous Volunteer</dc:contributor>\n\t\t<meta property=\"file-as\" refines=\"#transcriber-{i}\">Anonymous Volunteer, An</meta>\n"
 				else:
 					producers_xhtml = producers_xhtml + f"\t\t<dc:contributor id=\"transcriber-{i}\">{producer.strip('.')}</dc:contributor>\n\t\t<meta property=\"file-as\" refines=\"#transcriber-{i}\">TRANSCRIBER_SORT</meta>\n"
 
+				# Homepage
+				if "Distributed Proofread" in producer:
+					producers_xhtml = producers_xhtml + f"\t\t<meta property=\"se:url.homepage\" refines=\"#transcriber-{i}\">https://pgdp.net</meta>\n"
+
+				# LCCN
+				if "David Widger" in producer:
+					producers_xhtml = producers_xhtml + f"\t\t<meta property=\"se:url.authority.nacoaf\" refines=\"#transcriber-{i}\">http://id.loc.gov/authorities/names/no2011017869</meta>\n"
+
+				# Role
 				producers_xhtml = producers_xhtml + f"\t\t<meta property=\"role\" refines=\"#transcriber-{i}\" scheme=\"marc:relators\">trc</meta>\n"
 
 				i = i + 1
@@ -858,7 +896,7 @@ def _create_draft(args: Namespace):
 		else:
 			metadata_xml = regex.sub(r"<dc:contributor id=\"illustrator\">.+?scheme=\"marc:relators\">ill</meta>\n\t\t", "", metadata_xml, flags=regex.DOTALL)
 
-		if args.pg_url:
+		if args.pg_id:
 			if pg_subjects:
 				subject_xhtml = ""
 
@@ -873,7 +911,7 @@ def _create_draft(args: Namespace):
 
 					# Now, get the LCSH ID by querying LCSH directly.
 					try:
-						response = requests.get(f"https://id.loc.gov/search/?q=%22{urllib.parse.quote(subject)}%22")
+						response = requests.get(f"https://id.loc.gov/search/?q=cs:http://id.loc.gov/authorities/subjects&q=\"{urllib.parse.quote(subject)}\"")
 						result = regex.search(fr"<a title=\"Click to view record\" href=\"/authorities/subjects/([^\"]+?)\">{regex.escape(subject.replace(' -- ', '--'))}</a>", response.text)
 
 						loc_id = "Unknown"
@@ -892,7 +930,7 @@ def _create_draft(args: Namespace):
 				metadata_xml = regex.sub(r"\t\t<dc:subject id=\"subject-1\">SUBJECT_1</dc:subject>\s*<dc:subject id=\"subject-2\">SUBJECT_2</dc:subject>\s*<meta property=\"authority\" refines=\"#subject-1\">LCSH</meta>\s*<meta property=\"term\" refines=\"#subject-1\">LCSH_ID_1</meta>\s*<meta property=\"authority\" refines=\"#subject-2\">LCSH</meta>\s*<meta property=\"term\" refines=\"#subject-2\">LCSH_ID_2</meta>", "\t\t" + subject_xhtml.strip(), metadata_xml)
 
 			metadata_xml = metadata_xml.replace("<dc:language>LANG</dc:language>", f"<dc:language>{pg_language}</dc:language>")
-			metadata_xml = metadata_xml.replace("<dc:source>PG_URL</dc:source>", f"<dc:source>{args.pg_url}</dc:source>")
+			metadata_xml = metadata_xml.replace("<dc:source>PG_URL</dc:source>", f"<dc:source>{pg_url}</dc:source>")
 
 		file.seek(0)
 		file.write(metadata_xml)
@@ -905,10 +943,10 @@ def _create_draft(args: Namespace):
 		with repo.config_writer() as config:
 			config.set_value("user", "email", args.email)
 
-	if args.pg_url and pg_ebook_html and not is_pg_html_parsed:
+	if args.pg_id and pg_ebook_html and not is_pg_html_parsed:
 		raise se.InvalidXhtmlException("Couldn’t parse Project Gutenberg ebook source. This is usually due to invalid HTML in the ebook.")
 
-def create_draft() -> int:
+def create_draft(plain_output: bool) -> int:
 	"""
 	Entry point for `se create-draft`
 	"""
@@ -916,28 +954,25 @@ def create_draft() -> int:
 	parser = argparse.ArgumentParser(description="Create a skeleton of a new Standard Ebook in the current directory.")
 	parser.add_argument("-i", "--illustrator", dest="illustrator", nargs="+", help="an illustrator of the ebook")
 	parser.add_argument("-r", "--translator", dest="translator", nargs="+", help="a translator of the ebook")
-	parser.add_argument("-p", "--pg-url", dest="pg_url", help="the URL of the Project Gutenberg ebook to download")
+	parser.add_argument("-p", "--pg-id", dest="pg_id", type=se.is_positive_integer, help="the Project Gutenberg ID number of the ebook to download")
 	parser.add_argument("-e", "--email", dest="email", help="use this email address as the main committer for the local Git repository")
 	parser.add_argument("-o", "--offline", dest="offline", action="store_true", help="create draft without network access")
 	parser.add_argument("-a", "--author", dest="author", required=True, nargs="+", help="an author of the ebook")
 	parser.add_argument("-t", "--title", dest="title", required=True, help="the title of the ebook")
+	parser.add_argument("-w", "--white-label", action="store_true", help="create a generic epub skeleton without S.E. branding")
 	args = parser.parse_args()
 
-	if args.pg_url and not regex.match("^https?://www.gutenberg.org/ebooks/[0-9]+$", args.pg_url):
-		se.print_error("Project Gutenberg URL must look like: [url]https://www.gutenberg.org/ebooks/<EBOOK-ID>[url].")
-		return se.InvalidArgumentsException.code
-
 	try:
 		# Before we continue, confirm that there isn't a subtitle passed in with the title
 		if ":" in args.title:
 			console = Console(highlight=False, theme=se.RICH_THEME) # Syntax highlighting will do weird things when printing paths; force_terminal prints colors when called from GNU Parallel
-			console.print("Titles should not include a subtitle, as subtitles are separate metadata elements in [path]content.opf[/]. Are you sure you want to continue? \\[y/N]")
+			console.print(se.prep_output("Titles should not include a subtitle, as subtitles are separate metadata elements in [path]content.opf[/]. Are you sure you want to continue? \\[y/N]", plain_output))
 			if input().lower() not in {"yes", "y"}:
 				return se.InvalidInputException.code
 
 		_create_draft(args)
 	except se.SeException as ex:
-		se.print_error(ex)
+		se.print_error(ex, plain_output=plain_output)
 		return ex.code
 
 	return 0
