@@ -47,12 +47,12 @@ def _is_mobi(file_bytes: bytes) -> bool:
 
 	return file_bytes[:78][0x3C:0x3C+8] in (b"BOOKMOBI", b"TEXtREAd")
 
-def extract_ebook() -> int:
+def extract_ebook(plain_output: bool) -> int:
 	"""
 	Entry point for `se extract-ebook`
 	"""
 
-	parser = argparse.ArgumentParser(description="Extract an epub, mobi, or azw3 ebook into ./FILENAME.extracted/ or a target directory.")
+	parser = argparse.ArgumentParser(description="Extract an .epub, .mobi, or .azw3 ebook into ./FILENAME.extracted/ or a target directory.")
 	parser.add_argument("-o", "--output-dir", type=str, help="a target directory to extract into")
 	parser.add_argument("-v", "--verbose", action="store_true", help="increase output verbosity")
 	parser.add_argument("targets", metavar="TARGET", nargs="+", help="an epub, mobi, or azw3 file")
@@ -64,10 +64,10 @@ def extract_ebook() -> int:
 		target = Path(target).resolve()
 
 		if args.verbose:
-			console.print(f"Processing [path][link=file://{target}]{target}[/][/] ...", end="")
+			console.print(se.prep_output(f"Processing [path][link=file://{target}]{target}[/][/] ...", plain_output), end="")
 
 		if not path.isfile(target):
-			se.print_error(f"Not a file: [path][link=file://{target}]{target}[/][/].")
+			se.print_error(f"Not a file: [path][link=file://{target}]{target}[/][/].", plain_output=plain_output)
 			return se.InvalidInputException.code
 
 		if args.output_dir is None:
@@ -76,7 +76,7 @@ def extract_ebook() -> int:
 			extracted_path = Path(args.output_dir)
 
 		if extracted_path.exists():
-			se.print_error(f"Directory already exists: [path][link=file://{extracted_path}]{extracted_path}[/][/].")
+			se.print_error(f"Directory already exists: [path][link=file://{extracted_path}]{extracted_path}[/][/].", plain_output=plain_output)
 			return se.FileExistsException.code
 
 		with open(target, "rb") as binary_file:
