@@ -15,14 +15,12 @@ from rich.text import Text
 import se
 from se.se_epub import SeEpub
 
-def lint() -> int:
+def lint(plain_output: bool) -> int:
 	"""
 	Entry point for `se lint`
 	"""
 
 	parser = argparse.ArgumentParser(description="Check for various Standard Ebooks style errors.")
-	parser.add_argument("-n", "--no-colors", dest="colors", action="store_false", help="don’t use color or hyperlinks in output")
-	parser.add_argument("-p", "--plain", action="store_true", help="print plain text output, without tables or colors")
 	parser.add_argument("-s", "--skip-lint-ignore", action="store_true", help="ignore rules in se-lint-ignore.xml file")
 	parser.add_argument("-v", "--verbose", action="store_true", help="increase output verbosity")
 	parser.add_argument("directories", metavar="DIRECTORY", nargs="+", help="a Standard Ebooks source directory")
@@ -36,7 +34,7 @@ def lint() -> int:
 	# Rich needs to know the terminal width in order to format tables.
 	# If we're called from Parallel, there is no width because Parallel is not a terminal. Thus we must export $COLUMNS before
 	# invoking Parallel, and then get that value here.
-	console = Console(width=int(os.environ['COLUMNS']) if called_from_parallel and "COLUMNS" in os.environ else None, highlight=False, theme=se.RICH_THEME, force_terminal=force_terminal) # Syntax highlighting will do weird things when printing paths; force_terminal prints colors when called from GNU Parallel
+	console = Console(width=int(os.environ["COLUMNS"]) if called_from_parallel and "COLUMNS" in os.environ else None, highlight=False, theme=se.RICH_THEME, force_terminal=force_terminal) # Syntax highlighting will do weird things when printing paths; force_terminal prints colors when called from GNU Parallel
 
 	for directory in args.directories:
 		directory = Path(directory).resolve()
@@ -64,30 +62,29 @@ def lint() -> int:
 		# Print the table header
 		if ((len(args.directories) > 1 or called_from_parallel) and (messages or exception)) or args.verbose:
 			has_output = True
-			if args.plain:
+			if plain_output:
 				console.print(directory)
 			else:
-				console.print(f"[reverse]{directory}[/reverse]")
+				console.print(f"[reverse][path][link=file://{directory}]{directory}[/][/][/reverse]")
 
 		if exception:
 			has_output = True
-			se.print_error(exception)
+			se.print_error(exception, plain_output=plain_output)
 
 		# Print the tables
 		if messages:
 			has_output = True
 			return_code = se.LintFailedException.code
 
-			if args.plain:
+			if plain_output:
 				for message in messages:
-					label = "Manual Review:"
+					label = "[Manual Review]"
 
 					if message.message_type == se.MESSAGE_TYPE_ERROR:
-						label = "Error:"
+						label = "[Error]"
 
 					# Replace color markup with `
-					message.text = regex.sub(r"\[(?:/|xhtml|xml|val|attr|css|val|class|path|url|text|bash|link)(?:=[^\]]*?)*\]", "`", message.text)
-					message.text = regex.sub(r"`+", "`", message.text)
+					message.text = se.prep_output(message.text, True)
 
 					message_filename = ""
 					if message.filename:
@@ -101,40 +98,22 @@ def lint() -> int:
 							console.print(regex.sub(r"^", "\t", submessage, flags=regex.MULTILINE))
 			else:
 				for message in messages:
-					alert = "Manual Review"
+					alert = "[bright_yellow]Manual Review[/bright_yellow]"
 
 					if message.message_type == se.MESSAGE_TYPE_ERROR:
-						alert = "Error"
+						alert = "[bright_red]Error[/bright_red]"
 
-					message_text = message.text
+					# Add hyperlinks around message filenames
+					message_filename = ""
+					if message.filename:
+						message_filename = f"[link=file://{message.filename.resolve()}]{message.filename.name}[/link]"
 
-					if args.colors:
-						if message.message_type == se.MESSAGE_TYPE_ERROR:
-							alert = f"[bright_red]{alert}[/bright_red]"
-						else:
-							alert = f"[bright_yellow]{alert}[/bright_yellow]"
-
-						# Add hyperlinks around message filenames
-						message_filename = ""
-						if message.filename:
-							message_filename = f"[link=file://{message.filename.resolve()}]{message.filename.name}[/link]"
-					else:
-						# Replace color markup with `
-						message_text = regex.sub(r"\[(?:/|xhtml|xml|val|attr|css|val|class|path|url|text|bash|link)(?:=[^\]]*?)*\]", "`", message_text)
-						message_text = regex.sub(r"`+", "`", message_text)
-						message_filename = ""
-						if message.filename:
-							message_filename = message.filename.name
-
-					table_data.append([message.code, alert, message_filename, message_text])
+					table_data.append([message.code, alert, message_filename, message.text])
 
 					if message.submessages:
 						for submessage in message.submessages:
 							# Brackets don't need to be escaped in submessages if we instantiate them in Text()
-							if args.colors:
-								submessage_object = Text(submessage, style="dim")
-							else:
-								submessage_object = Text(submessage)
+							submessage_object = Text(submessage, style="dim")
 
 							table_data.append([" ", " ", Text("→", justify="right"), submessage_object])
 
@@ -150,11 +129,11 @@ def lint() -> int:
 				console.print(table)
 
 		if args.verbose and not messages and not exception:
-			if args.plain:
+			if plain_output:
 				console.print("OK")
 			else:
 				table = Table(show_header=False, box=box.SQUARE)
-				table.add_column("", style="white on green4 bold" if args.colors else None)
+				table.add_column("", style="white on green4 bold")
 				table.add_row("OK")
 				console.print(table)
 
