@@ -11,8 +11,9 @@ import regex
 from lxml import cssselect, etree
 from cssselect import parser
 
-import se
-import se.css
+# Not sure how to get around pylint error, so just ignore it for now
+# until someone can solve it
+import se.css # pylint: disable=cyclic-import
 
 CSS_SELECTOR_CACHE: Dict[str, cssselect.CSSSelector] = {}
 CSS_RULES_CACHE: Dict[str, List[se.css.CssRule]] = {}
@@ -40,11 +41,35 @@ class EasyXmlTree:
 	"""
 	A helper class to make some lxml operations a little less painful.
 	Represents an entire lxml tree.
+
+	This is not a complete XML parser. It only works if namespaces are only declared on the root element.
 	"""
 
-	def __init__(self, xml_string: str):
-		self.namespaces = {"re": "http://exslt.org/regular-expressions"} # Enable regular expressions in xpath
-		self.etree = etree.fromstring(str.encode(xml_string))
+	def __init__(self, xml: Union[str, etree._ElementTree]):
+		self.namespaces = {"re": "http://exslt.org/regular-expressions", "xml": "http://www.w3.org/XML/1998/namespace"} # Enable regular expressions in xpath; xml is the default xml namespace
+		self.default_namespace = None
+
+		if isinstance(xml, etree._ElementTree):
+			xml_string = etree.tostring(xml, encoding="unicode", with_tail=False)
+		else:
+			xml_string = xml
+
+		# Save the default namespace for later
+		for namespace in regex.findall(r" xmlns=\"([^\"]+?)\"", xml_string):
+			self.default_namespace = namespace
+
+		# Always remove the default namespaces, otherwise xpath with lxml is a huge pain
+		xml_string = regex.sub(r" xmlns=\"[^\"]+?\"", "", xml_string)
+
+		# Add additional namespaces we may have
+		for match in regex.findall(r" xmlns:(.+?)=\"([^\"]+?)\"", xml_string):
+			self.namespaces[match[0]] = match[1]
+
+		try:
+			self.etree = etree.fromstring(str.encode(xml_string))
+		except etree.XMLSyntaxError as ex:
+			raise se.InvalidXmlException(f"Couldn’t parse XML. Exception: {ex}") from ex
+
 		self.is_css_applied = False
 
 	def css_select(self, selector: str):
@@ -55,7 +80,7 @@ class EasyXmlTree:
 		try:
 			sel = CSS_SELECTOR_CACHE.get(selector)
 			if not sel:
-				sel = cssselect.CSSSelector(selector, translator="xhtml", namespaces={"xhtml": "http://www.w3.org/1999/xhtml", "epub": "http://www.idpf.org/2007/ops"})
+				sel = cssselect.CSSSelector(selector, translator="xhtml", namespaces=self.namespaces)
 				CSS_SELECTOR_CACHE[selector] = sel
 
 			return self.xpath(sel.path)
@@ -67,18 +92,21 @@ class EasyXmlTree:
 		Shortcut to select elements based on xpath selector.
 
 		If return_string is true, return a single string value instead of a list.
-
-		Warning: lxml has no support for an element without a namepace.  So, when using xpath or css_select, make sure to include a bogus namespace if necessary.
-		For example, in content.opf we can't do xpath("//metadata").  We have to use a bogus namespace: xpath("//opf:metadata")
 		"""
 
 		result: List[Union[str, EasyXmlElement]] = []
 
-		for element in self.etree.xpath(selector, namespaces=self.namespaces):
-			if isinstance(element, str):
-				result.append(element)
-			else:
-				result.append(EasyXmlElement(element, self.namespaces))
+		try:
+			for element in self.etree.xpath(selector, namespaces=self.namespaces):
+				if isinstance(element, str):
+					result.append(element)
+				else:
+					result.append(EasyXmlElement(element, self.namespaces))
+		except etree.XPathEvalError as ex:
+			# If we ask for an undefined namespace prefix, just return nothing
+			# instead of crashing
+			if str(ex) != "Undefined namespace prefix":
+				raise ex
 
 		if return_string and result:
 			return str(result[0])
@@ -154,115 +182,52 @@ class EasyXmlTree:
 					if attr.startswith("data-css-"):
 						node.remove_attr(attr)
 
-		xml = """<?xml version="1.0" encoding="utf-8"?>\n""" + etree.tostring(self.etree, encoding="unicode") + "\n"
+		xml = etree.tostring(self.etree, encoding="unicode")
+
+		# Re-insert the default namespace if we removed it earlier
+		if self.default_namespace:
+			xml = regex.sub(r"^<([a-z0-9\-]+)\b", fr'<\1 xmlns="{self.default_namespace}"', xml)
+
+		xml = """<?xml version="1.0" encoding="utf-8"?>\n""" + xml + "\n"
 
 		# Normalize unicode characters
 		xml = unicodedata.normalize("NFC", xml)
 
 		return xml
 
-class EasyContainerTree(EasyXmlTree):
-	"""
-	Wrapper for the container namespace.
-	"""
-
-	def __init__(self, xml_string: str):
-		# We have to remove the default namespace declaration from our document, otherwise
-		# xpath won't find anything at all. See http://stackoverflow.com/questions/297239/why-doesnt-xpath-work-when-processing-an-xhtml-document-with-lxml-in-python
-
-		super().__init__(xml_string.replace(" xmlns=\"urn:oasis:names:tc:opendocument:xmlns:container\"", ""))
-
-	def to_string(self) -> str:
-		"""
-		Serialize the tree to a string.
-		"""
-
-		xml = EasyXmlTree.to_string(self)
-
-		xml = regex.sub(r"<container(?!:)", "<container xmlns=\"urn:oasis:names:tc:opendocument:xmlns:container\"", xml)
-
-		return xml
-
-class EasyXhtmlTree(EasyXmlTree):
-	"""
-	Wrapper for the XHTML namespace.
-	"""
-
-	def __init__(self, xml_string: str):
-		# We have to remove the default namespace declaration from our document, otherwise
-		# xpath won't find anything at all. See http://stackoverflow.com/questions/297239/why-doesnt-xpath-work-when-processing-an-xhtml-document-with-lxml-in-python
-
-		super().__init__(xml_string.replace(" xmlns=\"http://www.w3.org/1999/xhtml\"", ""))
-
-		self.namespaces = {**self.namespaces, **{"epub": "http://www.idpf.org/2007/ops", "m": "http://www.w3.org/1998/Math/MathML"}}
-
-	def to_string(self) -> str:
-		"""
-		Serialize the tree to a string.
-		"""
-
-		xml = EasyXmlTree.to_string(self)
-
-		xml = regex.sub(r"<html(?!:)", "<html xmlns=\"http://www.w3.org/1999/xhtml\"", xml)
-
-		return xml
-
-class EasySvgTree(EasyXmlTree):
-	"""
-	Wrapper for the SVG namespace.
-	"""
-
-	def __init__(self, xml_string: str):
-		# We have to remove the default namespace declaration from our document, otherwise
-		# xpath won't find anything at all. See http://stackoverflow.com/questions/297239/why-doesnt-xpath-work-when-processing-an-xhtml-document-with-lxml-in-python
-
-		super().__init__(xml_string.replace(" xmlns=\"http://www.w3.org/2000/svg\"", ""))
-
-		self.namespaces = {**self.namespaces, **{"xlink": "http://www.w3.org/1999/xlink"}}
-
-	def to_string(self) -> str:
-		"""
-		Serialize the tree to a string.
-		"""
-
-		xml = EasyXmlTree.to_string(self)
-
-		xml = xml.replace("<svg", "<svg xmlns=\"http://www.w3.org/2000/svg\"")
-
-		return xml
-
-class EasyOpfTree(EasyXmlTree):
-	"""
-	Wrapper for the OPF namespace.
-	"""
-
-	def __init__(self, xml_string: str):
-		# We have to remove the default namespace declaration from our document, otherwise
-		# xpath won't find anything at all. See http://stackoverflow.com/questions/297239/why-doesnt-xpath-work-when-processing-an-xhtml-document-with-lxml-in-python
-
-		super().__init__(xml_string.replace(" xmlns=\"http://www.idpf.org/2007/opf\"", ""))
-
-		self.namespaces = {**self.namespaces, **{"dc": "http://purl.org/dc/elements/1.1/"}}
-
-	def to_string(self) -> str:
-		"""
-		Serialize the tree to a string.
-		"""
-
-		xml = EasyXmlTree.to_string(self)
-
-		xml = xml.replace("<package", "<package xmlns=\"http://www.idpf.org/2007/opf\"")
-
-		return xml
-
 class EasyXmlElement:
 	"""
 	Represents an lxml element.
 	"""
 
-	def __init__(self, lxml_element, namespaces=None):
+	def __init__(self, lxml_element: Union[str, etree._ElementTree], namespaces=None):
 		self.namespaces = namespaces
-		self.lxml_element = lxml_element
+
+		if isinstance(lxml_element, str):
+			dom = se.easy_xml.EasyXmlTree(lxml_element)
+			self.lxml_element = dom.etree
+		else:
+			self.lxml_element = lxml_element
+
+	def _replace_shorthand_namespaces(self, value:str) -> str:
+		"""
+		Given a string starting with a shorthand namespace, return
+		the fully qualified namespace.
+
+		This is useful for passing to raw lxml operations as lxml doesn't understand
+		shorthand namespaces.
+
+		Example:
+		epub:type -> {http://www.idpf.org/2007/ops}:type
+		"""
+
+		output = value
+
+		if self.namespaces:
+			for name, identifier in self.namespaces.items():
+				output = regex.sub(fr"^{name}:", f"{{{identifier}}}", output)
+
+		return output
 
 	def to_tag_string(self) -> str:
 		"""
@@ -278,12 +243,18 @@ class EasyXmlElement:
 		for name, value in self.lxml_element.items():
 			# Exclude applied CSS
 			if not name.startswith("data-css-"):
-				attrs += f" {name}=\"{value}\""
+				# Replace long namespaces from lxml with shorthand
+				short_name = name
+				for namespace, identifier in self.namespaces.items():
+					short_name = short_name.replace(f"{{{identifier}}}", f"{namespace}:")
 
-		attrs = attrs.replace("{http://www.idpf.org/2007/ops}", "epub:")
-		attrs = attrs.replace("{http://www.w3.org/XML/1998/namespace}", "xml:")
+				attrs += f" {short_name}=\"{value}\""
 
-		return f"<{self.lxml_element.tag}{attrs}>"
+		tag_name = self.lxml_element.tag
+		for namespace, identifier in self.namespaces.items():
+			tag_name = tag_name.replace(f"{{{identifier}}}", f"{namespace}:")
+
+		return f"<{tag_name}{attrs}>"
 
 	def to_string(self) -> str:
 		"""
@@ -327,31 +298,63 @@ class EasyXmlElement:
 
 		return None
 
-	def remove_attr(self, attribute: str):
+	def remove_attr(self, attribute: str) -> None:
 		"""
 		Remove an attribute from this node.
 		"""
 
-		etree.strip_attributes(self.lxml_element, attribute)
+		try:
+			self.lxml_element.attrib.pop(self._replace_shorthand_namespaces(attribute))
+		except KeyError:
+			# If the attribute doesn't exist, just continue
+			pass
+
+	def add_attr_value(self, attribute: str, value: str) -> None:
+		"""
+		Add a space-separated attribute value to the target attribute.
+		If the attribute doesn't exist, add it.
+
+		Mainly useful for HTML `class` attributes and epub `epub:type` attributes.
+
+		Example adding value `bar` to the `class` attribute:
+		<p class="foo"> -> <p class="foo bar">
+		"""
+
+		existing_value = self.get_attr(attribute) or ""
+
+		self.set_attr(attribute, (existing_value + " " + value).strip())
+
+	def remove_attr_value(self, attribute: str, value: str) -> None:
+		"""
+		Remove a space-separated attribute value from the target attribute.
+		If removing the value makes the attribute empty, remove the attribute.
+
+		Mainly useful for HTML `class` attributes and epub `epub:type` attributes.
+
+		Example removing value `bar` from the `class` attribute:
+		<p class="foo bar"> -> <p class="foo">
+		"""
+
+		if self.get_attr(attribute):
+			self.set_attr(attribute, regex.sub(fr"\s*\b{regex.escape(value)}\b\s*", "", self.get_attr(attribute)))
+
+			# If the attribute is now empty, remove it
+			if not self.get_attr(attribute):
+				self.remove_attr(attribute)
 
 	def get_attr(self, attribute: str) -> str:
 		"""
 		Return the value of an attribute on this element.
 		"""
 
-		attribute = attribute.replace("epub:", "{http://www.idpf.org/2007/ops}")
-		attribute = attribute.replace("xml:", "{http://www.w3.org/XML/1998/namespace}")
-
-		return self.lxml_element.get(attribute)
+		return self.lxml_element.get(self._replace_shorthand_namespaces(attribute))
 
 	def set_attr(self, attribute: str, value: str) -> str:
 		"""
 		Set the value of an attribute on this element.
 		"""
 
-		attribute = attribute.replace("epub:", "{http://www.idpf.org/2007/ops}")
-
-		return self.lxml_element.set(attribute, value)
+		return self.lxml_element.set(self._replace_shorthand_namespaces(attribute), value)
 
 	def xpath(self, selector: str, return_string: bool = False):
 		"""
@@ -413,12 +416,20 @@ class EasyXmlElement:
 		if self.lxml_element.tail:
 			prev = self.lxml_element.getprevious()
 			if prev is not None: # We can't do `if prev` because we get a FutureWarning from lxml
-				prev.tail = (prev.tail or '') + self.lxml_element.tail
+				prev.tail = (prev.tail or "") + self.lxml_element.tail
 			else:
-				parent.text = (parent.text or '') + self.lxml_element.tail
+				parent.text = (parent.text or "") + self.lxml_element.tail
 
 		parent.remove(self.lxml_element)
 
+	def wrap_with(self, node) -> None:
+		"""
+		Wrap this node in the passed node.
+		"""
+
+		self.lxml_element.addprevious(node.lxml_element)
+		node.lxml_element.insert(0, self.lxml_element)
+
 	def unwrap(self) -> None:
 		"""
 		Remove the element's wrapping tag and replace it with the element's contents.
@@ -445,10 +456,11 @@ class EasyXmlElement:
 		if self.lxml_element.text:
 			prev = self.lxml_element.getprevious()
 			if prev is None:
-				if parent.text:
-					parent.text = parent.text + self.lxml_element.text
-				else:
-					parent.text = self.lxml_element.text
+				if parent is not None:
+					if parent.text:
+						parent.text = parent.text + self.lxml_element.text
+					else:
+						parent.text = self.lxml_element.text
 			else:
 				if prev.tail:
 					prev.tail = prev.tail + self.lxml_element.text
@@ -458,12 +470,35 @@ class EasyXmlElement:
 		# This calls the EasyXmlTree.remove() function, not an lxml function
 		self.remove()
 
+	def replace_outer(self, node) -> None:
+		"""
+		Replace this node's wrapping element with the wrapping element of the passed
+		node, but keep this node's children.
+
+		Example:
+		<p>foo <b>bar</b></p> -> <div class="baz">foo <b>bar</b></div>
+		"""
+
+		node.lxml_element.tail = self.lxml_element.tail
+		node.lxml_element.text = self.lxml_element.text
+		node.children = self.children
+
+		self.lxml_element.text = ""
+		self.lxml_element.tail = ""
+		self.lxml_element.addnext(node.lxml_element)
+		self.unwrap()
+
 	def replace_with(self, node) -> None:
 		"""
 		Remove this node and replace it with the passed node
 		"""
 
-		self.lxml_element.addnext(node.lxml_element)
+		# lxml.addnext() moves this element's tail to the new element
+		if isinstance(node, EasyXmlElement):
+			self.lxml_element.addnext(node.lxml_element)
+		else:
+			self.lxml_element.addnext(node)
+
 		self.remove()
 
 	def append(self, node) -> None:
@@ -473,10 +508,67 @@ class EasyXmlElement:
 
 		if isinstance(node, EasyXmlElement):
 			self.lxml_element.append(node.lxml_element)
-
 		else:
 			self.lxml_element.append(node)
 
+	def prepend(self, node) -> None:
+		"""
+		Place node as the first child of this node.
+		"""
+
+		# If the node we're inserting in to has text, lxml will insert the new
+		# node *after* the text. So, we have to make the node's lxml `.text`
+		# the new node's lxml `.tail`.
+		target = node
+		if isinstance(node, EasyXmlElement):
+			target = node.lxml_element
+
+		if self.lxml_element.text:
+			if target.tail:
+				target.tail = target.tail + self.lxml_element.text
+			else:
+				target.tail = self.lxml_element.text
+
+		self.lxml_element.insert(0, target)
+		self.lxml_element.text = ""
+
+	def set_text(self, string: str) -> None:
+		"""
+		Replace all contents of this node, including text and any child nodes, with a text string.
+		"""
+
+		for child in list(self.lxml_element):
+			self.lxml_element.remove(child)
+
+		self.lxml_element.text = string
+
+	@property
+	def children(self) -> List:
+		"""
+		Return a list representing of this node's direct children
+		"""
+		children = []
+
+		for child in self.lxml_element:
+			children.append(EasyXmlElement(child))
+
+		return children
+
+	@children.setter
+	def children(self, children) -> None:
+		"""
+		Set the node's children.
+		"""
+
+		for child in self.lxml_element:
+			self.lxml_element.remove(child)
+
+		for child in children:
+			if isinstance(child, EasyXmlElement):
+				self.lxml_element.append(child.lxml_element)
+			else:
+				self.lxml_element.append(child)
+
 	@property
 	def tag(self) -> str:
 		"""
@@ -504,3 +596,50 @@ class EasyXmlElement:
 		"""
 
 		return self.lxml_element.text
+
+	@text.setter
+	def text(self, value: str) -> None:
+		"""
+		Set the lxml text attribute (the text up to the first child element)
+		"""
+
+		self.lxml_element.text = value
+
+	@property
+	def tail(self) -> str:
+		"""
+		Return only returns the text after this node
+
+		Example:
+		`<p class="test">Hello there!</p>` -> ``
+		`<p class="test">Hello there!</p> he said.` -> ` he said.`
+		"""
+
+		return self.lxml_element.tail
+
+	@tail.setter
+	def tail(self, value: str) -> None:
+		"""
+		Set the lxml tail attribute (the text after this element)
+		"""
+
+		self.lxml_element.tail = value
+
+	@property
+	def attrs(self) -> Dict:
+		"""
+		Return a dict of attributes for this node
+		"""
+
+		return self.lxml_element.attrib
+
+	@attrs.setter
+	def attrs(self, value: Dict) -> None:
+		"""
+		Return a dict of attributes for this node
+		"""
+
+		self.lxml_element.attrib.clear()
+
+		for name, val in sorted(value.items()):
+			self.set_attr(name, val)
