@@ -10,15 +10,17 @@ import math
 import string
 import unicodedata
 from pathlib import Path
+from typing import Union
 
 import regex
 import roman
 import tinycss2
 from lxml import etree
 from titlecase import titlecase as pip_titlecase
-from se.easy_xml import EasyXhtmlTree
+from unidecode import unidecode
 
 import se
+from se.easy_xml import EasyXmlTree
 
 
 # This list of phrasing tags is not intended to be exhaustive. The list is only used
@@ -73,7 +75,9 @@ def semanticate(xhtml: str) -> str:
 	xhtml = regex.sub(r"(?<!\<abbr\>)Mlles\.", r"<abbr>Mlles.</abbr>", xhtml)
 	xhtml = regex.sub(r"(?<!\<abbr\>)Messrs\.", r"<abbr>Messrs.</abbr>", xhtml)
 	xhtml = regex.sub(r"(?<!\<abbr\>)Messers\.", r"<abbr>Messers.</abbr>", xhtml)
-	xhtml = regex.sub(r"(?<!\<abbr[^\>]*?\>)(P\.(?:P\.)?S\.(?:S\.)?)", r"""<abbr class="initialism">\1</abbr>""", xhtml)
+	xhtml = regex.sub(r"\b(?<!\<abbr\>)([Vv])ol(s?)\.", r"<abbr>\1ol\2.</abbr>", xhtml)
+	xhtml = regex.sub(r"\b(?<!\<abbr\>)([Cc])hap\. ([0-9])", r"<abbr>\1hap.</abbr> \2", xhtml) # The number allows us to avoid phrases like `Hello, old chap.`
+	xhtml = regex.sub(r"(?<!\<abbr[^\>]*?\>)(P\.(?:P\.)?S\.(?:S\.)?)", r"""<abbr epub:type="z3998:initialism">\1</abbr>""", xhtml)
 	xhtml = regex.sub(r"(?<!\<abbr\>)Co\.", r"<abbr>Co.</abbr>", xhtml)
 	xhtml = regex.sub(r"(?<!\<abbr\>)Inc\.", r"<abbr>Inc.</abbr>", xhtml)
 	xhtml = regex.sub(r"(?<!\<abbr\>)Ltd\.", r"<abbr>Ltd.</abbr>", xhtml)
@@ -89,21 +93,23 @@ def semanticate(xhtml: str) -> str:
 	xhtml = regex.sub(r"(?<!\<abbr\>)(Jan\.|Feb\.|Mar\.|Apr\.|Jun\.|Jul\.|Aug\.|Sep\.|Sept\.|Oct\.|Nov\.|Dec\.)", r"<abbr>\1</abbr>", xhtml)
 	xhtml = regex.sub(r"(?<!\<abbr\>)No\.(\s+[0-9]+)", r"<abbr>No.</abbr>\1", xhtml)
 	xhtml = regex.sub(r"\b(?<!\<abbr\>)([Vv])s\.", r"<abbr>\1s.</abbr>", xhtml)
+	xhtml = regex.sub(r"\b(?<!\<abbr\>)([Ff])f\.", r"<abbr>\1f.</abbr>", xhtml) # ff. typically used in footnotes, means "and following"
+	xhtml = regex.sub(r"\b(?<!\<abbr\>)([Ll])ib\.", r"<abbr>\1ib.</abbr>", xhtml) # Lib. = Liber = Book
 	# python allows a variable lookbehind with the ( eoc)?; HOWEVER, it counts it as the
 	#	first capture group, so if there are one or more capture groups in the regex itself,
 	#	be sure to start at 2 when specifying the group(s) in the replacement spec.
-	xhtml = regex.sub(r"""(?<!\<abbr class="initialism( eoc)?"\>)([Ii])\.e\.""", r"""<abbr class="initialism">\2.e.</abbr>""", xhtml)
-	xhtml = regex.sub(r"""(?<!\<abbr class="initialism( eoc)?"\>)([Ee])\.g\.""", r"""<abbr class="initialism">\2.g.</abbr>""", xhtml)
-	xhtml = regex.sub(r"""(?<!\<abbr class="initialism( eoc)?"\>)\bN\.?B\.\b""", r"""<abbr class="initialism">N.B.</abbr>""", xhtml)
-	xhtml = regex.sub(r"""(?<!\<abbr class="degree( eoc)?"\>)Ph\.?\s*D\.?""", r"""<abbr class="degree">Ph. D.</abbr>""", xhtml)
-	xhtml = regex.sub(r"""(?<!\<abbr class="initialism( eoc)?"\>)I\.?O\.?U\.?\b""", r"""<abbr class="initialism">I.O.U.</abbr>""", xhtml)
-	xhtml = regex.sub(r"""(\s)(?<!\<abbr class="era( eoc)?"\>)A\.?D""", r"""\1<abbr class="era">AD</abbr>""", xhtml)
-	xhtml = regex.sub(r"""(\s)(?<!\<abbr class="era( eoc)?"\>)B\.?C""", r"""\1<abbr class="era">BC</abbr>""", xhtml)
-	xhtml = regex.sub(r"""(?<!\<abbr class="time( eoc)?"\>)([ap])\.\s?m\.""", r"""<abbr class="time">\2.m.</abbr>""", xhtml)
-	xhtml = regex.sub(r"""(?<!\<abbr class="name( eoc)?"\>)Thos\.""", r"""<abbr class="name">Thos.</abbr>""", xhtml)
-	xhtml = regex.sub(r"""(?<!\<abbr class="name( eoc)?"\>)Jas\.""", r"""<abbr class="name">Jas.</abbr>""", xhtml)
-	xhtml = regex.sub(r"""(?<!\<abbr class="name (eoc)?"\>)Chas\.""", r"""<abbr class="name">Chas.</abbr>""", xhtml)
-	xhtml = regex.sub(r"""(?<!\<abbr class="initialism( eoc)?"\>)\b([1-4]D)\b""", r"""<abbr class="initialism">\2</abbr>""", xhtml)
+	xhtml = regex.sub(r"""(?<!\<abbr epub:type="z3998:initialism"( class="eoc")?\>)([Ii])\.e\.""", r"""<abbr epub:type="z3998:initialism">\2.e.</abbr>""", xhtml)
+	xhtml = regex.sub(r"""(?<!\<abbr epub:type="z3998:initialism"( class="eoc")?\>)([Ee])\.g\.""", r"""<abbr epub:type="z3998:initialism">\2.g.</abbr>""", xhtml)
+	xhtml = regex.sub(r"""(?<!\<abbr epub:type="z3998:initialism"( class="eoc")?\>)\bN\.?B\.\b""", r"""<abbr epub:type="z3998:initialism">N.B.</abbr>""", xhtml)
+	xhtml = regex.sub(r"""(?<!\<abbr epub:type="z3998:name-title"( class="eoc")?\>)Ph\.?\s*D\.?""", r"""<abbr epub:type="z3998:name-title">Ph. D.</abbr>""", xhtml)
+	xhtml = regex.sub(r"""(?<!\<abbr epub:type="z3998:initialism"( class="eoc")?\>)I\.?O\.?U\.?\b""", r"""<abbr epub:type="z3998:initialism">I.O.U.</abbr>""", xhtml)
+	xhtml = regex.sub(r"""(\s)(?<!\<abbr epub:type="se:era"\>)A\.?D""", r"""\1<abbr epub:type="se:era">AD</abbr>""", xhtml)
+	xhtml = regex.sub(r"""(\s)(?<!\<abbr epub:type="se:era"( class="eoc")?\>)B\.?C""", r"""\1<abbr epub:type="se:era">BC</abbr>""", xhtml)
+	xhtml = regex.sub(r"""(?<!\<abbr( class="eoc")?\>)([ap])\.\s?m\.""", r"""<abbr>\2.m.</abbr>""", xhtml)
+	xhtml = regex.sub(r"""(?<!\<abbr epub:type="z3998:given-name"( class="eoc")?\>)(Thos\.|Jas\.|Chas\.)""", r"""<abbr epub:type="z3998:given-name">\2</abbr>""", xhtml)
+	xhtml = regex.sub(r"""(?<!\<abbr epub:type="z3998:initialism"( class="eoc")?\>)T\.?V\.?\b""", r"""<abbr epub:type="z3998:initialism">TV</abbr>""", xhtml)
+	xhtml = regex.sub(r"""(?<!\<abbr epub:type="z3998:initialism"( class="eoc")?\>)S\.?O\.?S\.?\b""", r"""<abbr epub:type="z3998:initialism">SOS</abbr>""", xhtml)
+	xhtml = regex.sub(r"""(?<!\<abbr epub:type="z3998:initialism"( class="eoc")?\>)\b([1-4]D)\b""", r"""<abbr epub:type="z3998:initialism">\2</abbr>""", xhtml)
 
 	# Wrap £sd shorthand
 	xhtml = regex.sub(r"([0-9½¼⅙⅚⅛⅜⅝]+)([sd⅞]\.)", r"\1<abbr>\2</abbr>", xhtml)
@@ -123,22 +129,26 @@ def semanticate(xhtml: str) -> str:
 
 	# Get Roman numerals >= 2 characters
 	# We only wrap these if they're standalone (i.e. not already wrapped in a tag) to prevent recursion in multiple runs
-	xhtml = regex.sub(r"([^\p{Letter}>])([ixvIXV]{2,})(\b|st\b|nd\b|rd\b|th\b)", r"""\1<span epub:type="z3998:roman">\2</span>\3""", xhtml)
+	# Ignore "numerals" followed by a dash, as they are more likely something like `x-ray` or `v-shaped`
+	xhtml = regex.sub(r"([^\p{Letter}>])([ixvIXV]{2,})(\b[^\-]|st\b|nd\b|rd\b|th\b)", r"""\1<span epub:type="z3998:roman">\2</span>\3""", xhtml)
 
 	# Get Roman numerals that are X or V and single characters.  We can't do I for obvious reasons.
-	xhtml = regex.sub(r"""([^\p{Letter}>\"])([vxVX])(\b|st\b|nd\b|rd\b|th\b)""", r"""\1<span epub:type="z3998:roman">\2</span>\3""", xhtml)
+	xhtml = regex.sub(r"""([^\p{Letter}>\"])([vxVX])(\b[^\-]|st\b|nd\b|rd\b|th\b)""", r"""\1<span epub:type="z3998:roman">\2</span>\3""", xhtml)
 
-	# We can assume a lowercase i is always a Roman numeral
-	xhtml = regex.sub(r"""([^\p{Letter}<>/\"])i\b""", r"""\1<span epub:type="z3998:roman">i</span>""", xhtml)
+	# We can assume a lowercase i is always a Roman numeral unless followed by ’
+	xhtml = regex.sub(r"""([^\p{Letter}<>/\"])i\b(?!’)""", r"""\1<span epub:type="z3998:roman">i</span>""", xhtml)
 
-	# Fix X-ray
-	xhtml = regex.sub(r"""<span epub:type="z3998:roman">([Xx])</span>-ray""", r"""\1-ray""", xhtml)
+	# Fix obscured names starting with I, V, or X
+	xhtml = regex.sub(fr"""<span epub:type="z3998:roman">([IVX])</span>{se.WORD_JOINER}⸺""", fr"""\1{se.WORD_JOINER}⸺""", xhtml)
 
 	# Add abbrevations around some SI measurements
 	xhtml = regex.sub(r"([0-9]+)\s*([cmk][mgl])\b", fr"\1{se.NO_BREAK_SPACE}<abbr>\2</abbr>", xhtml)
 
 	# Add abbrevations around Imperial measurements
-	xhtml = regex.sub(r"(?<![\$£0-9])([0-9]+)\s*(ft|in|yd|mi|pt|qt|gal|oz|lbs)\.?\b", fr"\1{se.NO_BREAK_SPACE}<abbr>\2.</abbr>", xhtml)
+	xhtml = regex.sub(r"(?<![\$£0-9,])([0-9½¼⅙⅚⅛⅜⅝⅞]+)\s*(ft|in|yd|mi|pt|qt|gal|oz|lbs)\.?\b", fr"\1{se.NO_BREAK_SPACE}<abbr>\2.</abbr>", xhtml)
+
+	# Fix some possible errors introduced by the above
+	xhtml = regex.sub(fr"((?:[Nn]o\.|[Nn]umber)\s[0-9]+){se.NO_BREAK_SPACE}<abbr>in\.</abbr>", r"\1 in", xhtml)
 
 	# Tweak some other Imperial measurements
 	xhtml = regex.sub(r"([0-9]+)\s*m\.?p\.?h\.?", fr"\1{se.NO_BREAK_SPACE}<abbr>mph</abbr>", xhtml, flags=regex.IGNORECASE)
@@ -536,10 +546,10 @@ def _format_style_elements(tree: etree.ElementTree):
 			css = format_css(node.text)
 
 			# Get the <style> element's indentation
-			indent = node.xpath("preceding-sibling::text()[1]")[0].replace("\n", "")
+			indent = node.xpath("./preceding-sibling::text()[1]")[0].replace("\n", "")
 
 			# Indent the CSS one level deeper than the <style> element
-			css = ''.join(indent + "\t" + line + "\n" for line in css.splitlines())
+			css = "".join(indent + "\t" + line + "\n" for line in css.splitlines())
 			css = css.strip("\n")
 			css = regex.sub(r"^\s+$", "", css, flags=regex.MULTILINE) # Remove indents from lines that are just white space
 
@@ -628,7 +638,7 @@ def format_xhtml(xhtml: str) -> str:
 	namespaces = {"xhtml": "http://www.w3.org/1999/xhtml", "epub": "http://www.idpf.org/2007/ops", "re": "http://exslt.org/regular-expressions"} # re enables regular expressions in xpath
 
 	# Epub3 doesn't allow named entities, so convert them to their unicode equivalents
-	# But, don't unescape the content.opf long-description accidentally
+	# But, don't unescape the metadata file long-description accidentally
 	xhtml = regex.sub(r"&#?\w+;", _replace_character_references, xhtml)
 
 	# Remove unnecessary doctypes which can cause xmllint to hang
@@ -715,6 +725,10 @@ def format_opf(xml: str) -> str:
 		# After composing the string, lxml adds namespaces to every tag. The only way to remove them is with regex.
 		xhtml = regex.sub(r"\sxmlns(:.+?)?=\"[^\"]+?\"", "", xhtml)
 
+		# Make some easy fixes
+		xhtml = regex.sub(r"<p>\s+", "<p>", xhtml)
+		xhtml = regex.sub(r"\s+</p>", "</p>", xhtml)
+
 		# Remove the children so that we can replace them with the escaped xhtml
 		for child in node:
 			node.remove(child)
@@ -848,6 +862,13 @@ def _format_css_component_list(content: list, in_selector=False, in_paren_block=
 	# Replace naked :pseudo-class selectors with *
 	output = output.replace(" :", " *:")
 
+	# Removing spaces after : may mess up media queries like:
+	# `@media all and (prefers-color-scheme: dark)` -> `@media all and (prefers-color-scheme:dark)`
+	# Here we try to re-add spaces after a : if it's within a paren block.
+	# We could do this during parsing but we would need to peek ahead to the next item in the loop which
+	# is too much trouble right now.
+	output = regex.sub(r"\(([^\"\s]+?):([^\"\s]+?)", r"(\1: \2", output)
+
 	return output.strip()
 
 def _format_css_rules(content: list, indent_level: int) -> str:
@@ -1034,9 +1055,9 @@ def titlecase(text: str) -> str:
 	# since they're typically lowercased anyway. (Except for things like `alt`, but we won't be titlecasing images!)
 	text = regex.sub(r"<(/?)([^>]+?)>", lambda result: "<" + result.group(1) + result.group(2).lower() + ">", text)
 
-	# Uppercase Roman numerals, but only if they are valid Roman numerals
+	# Uppercase Roman numerals, but only if they are valid Roman numerals and they are not "MIX" (which is much more likely to be an English word than a Roman numeral
 	try:
-		text = regex.sub(r"(\s)([ivxlcdm]+)(\s|$)", lambda result: result.group(1) + result.group(2).upper() + result.group(3) if roman.fromRoman(result.group(2).upper()) else result.group(2), text, flags=regex.IGNORECASE)
+		text = regex.sub(r"(\s)([ivxlcdm]+)(\b)", lambda result: result.group(1) + result.group(2).upper() + result.group(3) if result.group(2).upper() != "MIX" and roman.fromRoman(result.group(2).upper()) else result.group(1) + result.group(2), text, flags=regex.IGNORECASE)
 	except roman.InvalidRomanNumeralError:
 		pass
 
@@ -1057,9 +1078,12 @@ def titlecase(text: str) -> str:
 	# Lowercase "in", if followed by a semicolon (but not words like "inheritance")
 	text = regex.sub(r"\b; In\b", "; in", text)
 
-	# Lowercase th', sometimes used poetically
+	# Lowercase th’, sometimes used poetically
 	text = regex.sub(r"\b Th’ \b", " th’ ", text)
 
+	# Lowercase o’
+	text = regex.sub(r"\b O’ \b", " o’ ", text)
+
 	# Uppercase words that begin compound words, like "to-night" (which might appear in poetry)
 	text = regex.sub(r" ([\p{Lowercase_Letter}])([\p{Lowercase_Letter}]+\-)", lambda result: " " + result.group(1).upper() + result.group(2), text)
 
@@ -1113,6 +1137,9 @@ def titlecase(text: str) -> str:
 	# Uppercase initialisms
 	text = regex.sub(r"\b(([\p{Letter}]\.)+)", lambda result: result.group(1).upper(), text)
 
+	# Lowercase `mm` (millimeters, as in `50 mm gun`) unless it's followed by a period in which case it's likely `Mm.` (Monsieurs)
+	text = regex.sub(r"(\s)MM(\s|$)", r"\1mm\2", text)
+
 	# Fix html entities
 	text = text.replace("&Amp;", "&amp;")
 
@@ -1133,7 +1160,7 @@ def make_url_safe(text: str) -> str:
 	"""
 
 	# 1. Convert accented characters to unaccented characters
-	text = regex.sub(r"\p{M}", "", unicodedata.normalize("NFKD", text))
+	text = unidecode(text)
 
 	# 2. Trim
 	text = text.strip()
@@ -1206,9 +1233,6 @@ def simplify_css(css: str) -> str:
 	css = css.replace("{,", ",")
 	css = css.replace(",,", ",")
 
-	# Now replace abbr styles with spans, because ADE screws up with unrecognized elements
-	css = css.replace("abbr", "span")
-
 	# Replace shorthand CSS with longhand properties, another ADE screwup
 	css = regex.sub(r"margin:\s*([^\s]+?)\s*;", "margin-top: \\1;\n\tmargin-right: \\1;\n\tmargin-bottom: \\1;\n\tmargin-left: \\1;", css)
 	css = regex.sub(r"margin:\s*([^\s]+?)\s+([^\s]+?)\s*;", "margin-top: \\1;\n\tmargin-right: \\2;\n\tmargin-bottom: \\1;\n\tmargin-left: \\2;", css)
@@ -1219,6 +1243,9 @@ def simplify_css(css: str) -> str:
 	css = css.replace("all-small-caps;", "small-caps;\n\ttext-transform: lowercase;")
 	css = regex.sub(r"text-align:\s*initial\s*;", "text-align: left;", css)
 
+	# Include `all and` in @media queries, otherwise RMSDK will dump the entire stylesheet
+	css = regex.sub(r"@media\s*\(", "@media all and (", css)
+
 	# Replace CSS namespace selectors with classes
 	# For example, p[epub|type~="z3998:salutation"] becomes p.epub-type-z3998-salutation
 	for line in regex.findall(r"\[epub\|type\~\=\"[^\"]*?\"\]", css):
@@ -1228,7 +1255,7 @@ def simplify_css(css: str) -> str:
 	return css
 
 
-def generate_title(xhtml) -> str:
+def generate_title(xhtml: Union[str, EasyXmlTree]) -> str:
 	"""
 	Generate the value for the <title> tag of a string of XHTML, based on the rules in the SE manual.
 
@@ -1241,7 +1268,7 @@ def generate_title(xhtml) -> str:
 
 	try:
 		if isinstance(xhtml, str):
-			dom = EasyXhtmlTree(xhtml)
+			dom = EasyXmlTree(xhtml)
 		else:
 			dom = deepcopy(xhtml)
 	except Exception as ex:
@@ -1265,10 +1292,10 @@ def generate_title(xhtml) -> str:
 		if closest_parent_sections:
 			closest_parent_section = closest_parent_sections[0]
 		else:
-			raise se.InvalidSeEbookException("No [xhtml]<section>[/] or [xhtml]<article>[/] element for [xhtml]<hgroup>[/]")
+			raise se.InvalidSeEbookException("No [xhtml]<section>[/] or [xhtml]<article>[/] element for [xhtml]<hgroup>[/].")
 
 		# If the closest parent <section> or <article> is a part, division, or volume, then keep all <hgroup> children
-		if closest_parent_section.get_attr("epub:type") and ("part" not in closest_parent_section.get_attr("epub:type") and "division" not in closest_parent_section.get_attr("epub:type") and "volume" not in closest_parent_section.get_attr("epub:type")):
+		if not closest_parent_section.get_attr("epub:type") or (closest_parent_section.get_attr("epub:type") and ("part" not in closest_parent_section.get_attr("epub:type") and "division" not in closest_parent_section.get_attr("epub:type") and "volume" not in closest_parent_section.get_attr("epub:type"))):
 			# Else, if the closest parent <section> or <article> is a halftitlepage, then discard <hgroup> subtitles
 			if closest_parent_section.get_attr("epub:type") and "halftitlepage" in closest_parent_section.get_attr("epub:type"):
 				for node in hgroup_element.xpath("./*[contains(@epub:type, 'subtitle')]"):
@@ -1285,7 +1312,7 @@ def generate_title(xhtml) -> str:
 		try:
 			title = regex.sub(r"\s+", " ", hgroup_element.xpath("./*[1]")[0].inner_text().strip())
 		except Exception as ex:
-			raise se.InvalidSeEbookException("Couldn't find title in [xhml]<hgroup>[/]") from ex
+			raise se.InvalidSeEbookException("Couldn’t find title in [xhml]<hgroup>[/].") from ex
 
 		subtitle = hgroup_element.xpath("./*[2]")
 		if subtitle:
