@@ -105,8 +105,7 @@ def _color_to_alpha(image: Image, color=None) -> Image:
 	return new_image
 
 # Note: We can't type hint driver, because we conditionally import selenium for performance reasons
-# iBooks srcset bug: Temporarily ignore this line in pylint
-def render_mathml_to_png(driver, mathml: str, output_filename: Path, output_filename_2x: Path) -> None: # pylint: disable=unused-argument
+def render_mathml_to_png(driver, mathml: str, output_filename: Path, output_filename_2x: Path) -> None:
 	"""
 	Render a string of MathML into a transparent PNG file.
 
@@ -121,8 +120,8 @@ def render_mathml_to_png(driver, mathml: str, output_filename: Path, output_file
 	"""
 
 	# For some reason, we must use an .xhtml suffix. Without that, some mathml expressions don't render.
-	with tempfile.NamedTemporaryFile(mode="w+", suffix=".xhtml") as mathml_file:
-		with tempfile.NamedTemporaryFile(mode="w+", suffix=".png") as png_file:
+	with tempfile.NamedTemporaryFile(mode="w+", suffix=".xhtml", encoding="utf-8") as mathml_file:
+		with tempfile.NamedTemporaryFile(mode="w+", suffix=".png", encoding="utf-8") as png_file:
 			mathml_file.write(f"<?xml version=\"1.0\" encoding=\"utf-8\"?><html xmlns=\"http://www.w3.org/1999/xhtml\"><head><meta charset=\"utf-8\"/><title>MathML</title></head><body>{mathml}</body></html>")
 			mathml_file.seek(0)
 
@@ -138,9 +137,8 @@ def render_mathml_to_png(driver, mathml: str, output_filename: Path, output_file
 			image.save(output_filename_2x)
 
 			# Save normal version
-			# iBooks srcset bug: once srcset works in iBooks, uncomment these lines
-			# image = image.resize((image.width // 2, image.height // 2))
-			# image.save(output_filename)
+			image = image.resize((image.width // 2, image.height // 2))
+			image.save(output_filename)
 
 def remove_image_metadata(filename: Path) -> None:
 	"""
@@ -161,7 +159,7 @@ def remove_image_metadata(filename: Path) -> None:
 		# JPEG images are lossy, and PIL will recompress them on save.
 		# Instead of using PIL, read the byte stream and remove all metadata that way.
 		# Inspired by https://github.com/hMatoba/Piexif
-		with open(filename, 'rb+') as file:
+		with open(filename, "rb+") as file:
 			jpeg_data = file.read()
 
 			if jpeg_data[0:2] != b"\xff\xd8":
@@ -255,7 +253,7 @@ def svg_text_to_paths(in_svg: Path, out_svg: Path, remove_style=True) -> None:
 	"""
 
 	font_paths = []
-	name_list = {"league-spartan": ["league-spartan-bold.svg"], "sorts-mill-goudy": ["sorts-mill-goudy-italic.svg", "sorts-mill-goudy.svg"]}
+	name_list = {"league_spartan": ["league-spartan-bold.svg"], "sorts_mill_goudy": ["sorts-mill-goudy-italic.svg", "sorts-mill-goudy.svg"]}
 	for font_family, font_names in name_list.items():
 		for font_name in font_names:
 			with importlib_resources.path(f"se.data.fonts.{font_family}", font_name) as font_path:
@@ -269,7 +267,7 @@ def svg_text_to_paths(in_svg: Path, out_svg: Path, remove_style=True) -> None:
 	try:
 		xml = etree.fromstring(str.encode(svg_in_raw))
 	except Exception as ex:
-		raise se.InvalidXmlException(f"Couldn’t parse SVG file: [path][link={in_svg.resolve()}]{in_svg}[/][/]") from ex
+		raise se.InvalidXmlException(f"Couldn’t parse SVG file: [path][link={in_svg.resolve()}]{in_svg}[/][/].") from ex
 
 	svg_ns = "{http://www.w3.org/2000/svg}"
 
