@@ -8,7 +8,6 @@ the function is very big and it makes editing easier to put in a separate file.
 """
 
 from enum import Enum
-from pathlib import Path
 from typing import Tuple, List
 import regex
 from lxml import etree
@@ -50,6 +49,7 @@ class TocItem:
 	# pylint: disable=too-many-instance-attributes
 
 	file_link = ""
+	hidden = False # Did the <h#> element have the `hidden` attribute? If so, that means we MUST include it as a subtitle
 	level = 0
 	roman = ""
 	title = ""
@@ -75,7 +75,7 @@ class TocItem:
 
 		out_string = ""
 		if not self.title:
-			raise se.InvalidInputException(f"Couldn't find title in: [path][link=file://{self.file_link}]{self.file_link}[/][/].")
+			raise se.InvalidInputException(f"Couldn’t find title in: [path][link=file://{self.file_link}]{self.file_link}[/][/].")
 
 		if self.subtitle and self.lang:
 			# test for a foreign language subtitle, and adjust accordingly
@@ -90,7 +90,7 @@ class TocItem:
 				out_string += f"<a href=\"text/{self.file_link}\"><span epub:type=\"z3998:roman\">{self.roman}</span>: {self.subtitle}</a>\n"
 		else:
 			# title has text other than a roman numeral
-			if self.subtitle != "" and (self.title_is_ordinal or (self.division in [BookDivision.PART, BookDivision.DIVISION, BookDivision.VOLUME])):
+			if self.subtitle != "" and (self.hidden or self.title_is_ordinal or (self.division in [BookDivision.PART, BookDivision.DIVISION, BookDivision.VOLUME])):
 				# Use the subtitle only if we're a Part or Division or Volume or if title was an ordinal
 				out_string += f"<a href=\"text/{self.file_link}\">{self.title}"
 
@@ -108,7 +108,7 @@ class TocItem:
 
 		return out_string
 
-	def landmark_link(self, work_type: str = "fiction", work_title: str = "WORK_TITLE"):
+	def landmark_link(self, work_type: str = "fiction", work_title: str = "WORK_TITLE") -> str:
 		"""
 		Generates the landmark item (including list item tags) for the ToC item
 
@@ -156,7 +156,7 @@ def get_place(node: EasyXmlElement) -> Position:
 
 	return retval
 
-def add_landmark(dom: EasyXmlTree, textf: str, landmarks: list):
+def add_landmark(dom: EasyXmlTree, textf: str, landmarks: list) -> None:
 	"""
 	Adds an item to landmark list with appropriate details.
 
@@ -172,18 +172,23 @@ def add_landmark(dom: EasyXmlTree, textf: str, landmarks: list):
 	epub_type = ""
 	sections = dom.xpath("//body/*[name() = 'section' or name() = 'article']")
 	if not sections:
-		raise se.InvalidInputException("Couldn't locate first section")
+		raise se.InvalidInputException("Couldn’t locate first [xhtml]<section>[/] or [xhtml]<article>[/].")
 	epub_type = sections[0].get_attr("epub:type")
 	bodys = dom.xpath("//body")
 	if not bodys:
-		raise se.InvalidInputException("Couldn't locate body")
+		raise se.InvalidInputException("Couldn’t locate [xhtml]<body>[/].")
 
 	if not epub_type:  # some productions don't have an epub:type in outermost section, so get it from body tag
 		epub_type = bodys[0].get_attr("epub:type")
+		if not epub_type:
+			epub_type = ""
 
 	if epub_type in ["frontmatter", "bodymatter", "backmatter"]:
 		return  # if epub_type is ONLY frontmatter, bodymatter, backmatter, we don't want this as a landmark
 
+	# We may wind up with a (front|body|back)matter semantic in epub_type, remove it here since we add it to the landmark later
+	epub_type = regex.sub(r"(front|body|back)matter\s*", "", epub_type)
+
 	landmark = TocItem()
 	if epub_type:
 		landmark.epub_type = epub_type
@@ -198,7 +203,7 @@ def add_landmark(dom: EasyXmlTree, textf: str, landmarks: list):
 				landmark.title = landmark.epub_type.capitalize()
 		landmarks.append(landmark)
 
-def process_landmarks(landmarks_list: list, work_type: str, work_title: str):
+def process_landmarks(landmarks_list: list, work_type: str, work_title: str) -> str:
 	"""
 	Runs through all found landmark items and writes them to the toc file.
 
@@ -206,9 +211,6 @@ def process_landmarks(landmarks_list: list, work_type: str, work_title: str):
 	landmarks_list: the completed list of landmark items
 	work_type: "fiction" or "non-fiction"
 	work_title: the title of the book
-
-	OUTPUTS:
-	None
 	"""
 
 	# we don't want frontmatter items to be included once we've started the body items
@@ -298,8 +300,8 @@ def output_toc(item_list: list, landmark_list, toc_path: str, work_type: str, wo
 		raise se.InvalidInputException("Too few ToC items found.")
 
 	try:
-		with open(toc_path) as file:
-			toc_dom = se.easy_xml.EasyXhtmlTree(file.read())
+		with open(toc_path, "r", encoding="utf-8") as file:
+			toc_dom = se.easy_xml.EasyXmlTree(file.read())
 	except Exception as ex:
 		raise se.InvalidInputException(f"Existing ToC not found. Exception: {ex}")
 
@@ -362,7 +364,7 @@ def extract_strings(node: EasyXmlElement) -> str:
 	out_string = strip_notes(out_string)
 	return regex.sub(r"[\n\t]", "", out_string)
 
-def process_headings(dom: EasyXmlTree, textf: str, toc_list: list, nest_under_halftitle: bool, single_file: bool):
+def process_headings(dom: EasyXmlTree, textf: str, toc_list: list, nest_under_halftitle: bool, single_file: bool) -> None:
 	"""
 	Find headings in current file and extract title data
 	into items added to toc_list.
@@ -383,7 +385,7 @@ def process_headings(dom: EasyXmlTree, textf: str, toc_list: list, nest_under_ha
 	if body:
 		place = get_place(body[0])
 	else:
-		raise se.InvalidInputException("Couldn't locate body node")
+		raise se.InvalidInputException("Couldn’t locate [xhtml]<body>[/].")
 
 	is_toplevel = True
 
@@ -402,8 +404,9 @@ def process_headings(dom: EasyXmlTree, textf: str, toc_list: list, nest_under_ha
 		content_item = dom.xpath("//p | //header | //img")
 		if content_item is not None:
 			parents = content_item[0].xpath("./ancestor::*[name() = 'section' or name() = 'article']")
-			special_item.level = len(parents)
-			if special_item.level == 0:
+			if parents:
+				special_item.level = len(parents)
+			else:  # couldn't find a suitable parent, so just put it at top level
 				special_item.level = 1
 		if nest_under_halftitle:
 			special_item.level += 1
@@ -472,6 +475,9 @@ def process_a_heading(node: EasyXmlElement, textf: str, is_toplevel: bool, singl
 
 	toc_item.lang = node.get_attr("xml:lang")
 
+	if node.get_attr("hidden"):
+		toc_item.hidden = True
+
 	epub_type = node.get_attr("epub:type")
 
 	# it may be an empty header tag eg <h3>, so we pass its parent rather than itself to evaluate the parent's descendants
@@ -513,14 +519,14 @@ def get_child_strings(node: EasyXmlElement) -> str:
 	Get child strings
 	"""
 
-	children = node.xpath("*")
+	children = node.xpath("./*")
 	child_strs = ""
 	for child in children:
 		child_strs += child.to_string() + "\n"
 	return child_strs
 
 
-def evaluate_descendants(node: EasyXmlElement, toc_item):
+def evaluate_descendants(node: EasyXmlElement, toc_item: TocItem) -> TocItem:
 	"""
 	Burrow down into a hgroup structure to qualify the ToC item
 
@@ -535,6 +541,10 @@ def evaluate_descendants(node: EasyXmlElement, toc_item):
 		if not toc_item.lang:
 			toc_item.lang = child.get_attr("xml:lang")
 		epub_type = child.get_attr("epub:type")
+
+		if child.get_attr("hidden"):
+			toc_item.hidden = True
+
 		if not epub_type:
 			# should be a label/ordinal grouping
 			child_strings = get_child_strings(child)
@@ -548,6 +558,8 @@ def evaluate_descendants(node: EasyXmlElement, toc_item):
 				child_strings = regex.sub(r"\bordinal\b", "", child_strings)
 				# remove extra spaces
 				child_strings = regex.sub(r"[ ]{2,}", " ", child_strings)
+				# get rid of any endnotes
+				child_strings = strip_notes(child_strings)
 				toc_item.title = child_strings.strip()
 			continue  # skip the following
 		if "z3998:roman" in epub_type:
@@ -561,7 +573,7 @@ def evaluate_descendants(node: EasyXmlElement, toc_item):
 		if "subtitle" in epub_type:
 			toc_item.subtitle = extract_strings(child)
 		else:
-			if "title" in epub_type:  # this allows for 'fulltitle' to work here, too
+			if "title" in epub_type:  # this allows for `fulltitle` to work here, too
 				if toc_item.title or toc_item.roman or toc_item.title_is_ordinal:  # if title already filled, must be a subtitle
 					toc_item.subtitle = extract_strings(child)
 				else:
@@ -621,9 +633,9 @@ def strip_notes(text: str) -> str:
 	cleaned html string
 	"""
 
-	return regex.sub(r'<a[^>]*?epub:type="noteref"[^>]*?>.*?<\/a>', "", text)
+	return regex.sub(r"""<a[^>]*?epub:type="noteref"[^>]*?>.*?<\/a>""", "", text)
 
-def process_all_content(file_list: list, text_path: str) -> Tuple[list, list]:
+def process_all_content(file_list: list) -> Tuple[list, list]:
 	"""
 	Analyze the whole content of the project, build and return lists
 	if toc_items and landmarks.
@@ -642,14 +654,13 @@ def process_all_content(file_list: list, text_path: str) -> Tuple[list, list]:
 	# We make two passes through the work, because we need to know
 	# how many bodymatter items there are. So we do landmarks first.
 	for textf in file_list:
-		file_path = Path(text_path) / textf
 		try:
-			with open(file_path) as file:
-				dom = se.easy_xml.EasyXhtmlTree(file.read())
+			with open(textf, encoding="utf-8") as file:
+				dom = se.easy_xml.EasyXmlTree(file.read())
 		except Exception as ex:
-			raise se.InvalidFileException(f"Couldn’t open file: [path][link=file://{file_path}]{file_path}[/][/].") from ex
+			raise se.InvalidFileException(f"Couldn’t open file: [path][link=file://{textf}]{textf}[/][/]. Exception: {ex}") from ex
 
-		add_landmark(dom, textf, landmarks)
+		add_landmark(dom, textf.name, landmarks)
 
 	# Now we test to see if there is only one body item
 	body_items = [item for item in landmarks if item.place == Position.BODY]
@@ -658,17 +669,17 @@ def process_all_content(file_list: list, text_path: str) -> Tuple[list, list]:
 	nest_under_halftitle = False
 	place = Position.NONE
 	for textf in file_list:
-		with open(Path(text_path) / textf, "r", encoding="utf-8") as file:
-			dom = se.easy_xml.EasyXhtmlTree(file.read())
+		with open(textf, "r", encoding="utf-8") as file:
+			dom = se.easy_xml.EasyXmlTree(file.read())
 		body = dom.xpath("//body")
 		if body:
 			place = get_place(body[0])
 		else:
-			raise se.InvalidInputException("Couldn't locate body node")
+			raise se.InvalidInputException("Couldn’t locate [xhtml]<body>[/].")
 		if place == Position.BACK:
 			nest_under_halftitle = False
-		process_headings(dom, textf, toc_list, nest_under_halftitle, single_file)
-		if textf == "halftitle.xhtml":
+		process_headings(dom, textf.name, toc_list, nest_under_halftitle, single_file)
+		if dom.xpath("/html/body//*[contains(@epub:type, 'halftitlepage')]"):
 			nest_under_halftitle = True
 
 	# We add this dummy item because outputtoc always needs to look ahead to the next item.
@@ -684,10 +695,9 @@ def generate_toc(self) -> str:
 	Entry point for `SeEpub.generate_toc()`.
 	"""
 
-	file_list = self.get_content_files()
 	work_title = self.get_work_title()
 	work_type = self.get_work_type()
 
-	landmarks, toc_list = process_all_content(file_list, self.path / "src" / "epub" / "text")
+	landmarks, toc_list = process_all_content(self.spine_file_paths)
 
-	return output_toc(toc_list, landmarks, self.path / "src" / "epub" / "toc.xhtml", work_type, work_title)
+	return output_toc(toc_list, landmarks, self.toc_path, work_type, work_title)
