@@ -4,13 +4,14 @@ Defines various typography-related functions
 """
 
 import html
-from pathlib import Path
-from typing import Optional
+from typing import Optional, Union
+
 import pyphen
 import regex
 import smartypants
+
 import se
-from se.formatting import EasyXhtmlTree
+from se.formatting import EasyXmlTree
 
 
 def typogrify(xhtml: str, smart_quotes: bool = True) -> str:
@@ -76,6 +77,9 @@ def typogrify(xhtml: str, smart_quotes: bool = True) -> str:
 	# Some older texts use the ,— construct; remove that archaichism
 	xhtml = xhtml.replace(",—", "—")
 
+	# Remove shy hyphens
+	xhtml = xhtml.replace(se.SHY_HYPHEN, "")
+
 	# Fix some common em-dash transcription errors
 	xhtml = regex.sub(r"([:;])-([\p{Letter}])", r"\1—\2", xhtml, flags=regex.IGNORECASE)
 	xhtml = regex.sub(r"([\p{Letter}])-“", r"\1—“", xhtml, flags=regex.IGNORECASE)
@@ -101,9 +105,15 @@ def typogrify(xhtml: str, smart_quotes: bool = True) -> str:
 	# Remove word joiners from following opening tags--they're usually never correct
 	xhtml = regex.sub(fr"<([\p{{Letter}}]+)([^>]*?)>{se.WORD_JOINER}", r"<\1\2>", xhtml, flags=regex.IGNORECASE)
 
+	# Add a word joiner after em dashes within <cite> elements
+	xhtml = regex.sub(r"<cite([^>]*?)>—", fr"<cite\1>—{se.WORD_JOINER}", xhtml)
+
 	# Finally fix some other mistakes
 	xhtml = xhtml.replace("—-", "—")
 
+	# Possessives after inline elements need to be corrected after Smartypants
+	xhtml = regex.sub(r"</(i|em|b|strong|q|span)>‘(s|d)", r"</\1>’\2", xhtml)
+
 	# Replace two-em-dashes with an em-dash, but try to exclude ones being used for elision
 	xhtml = regex.sub(fr"([I\p{{Lowercase_Letter}}>\.]{se.WORD_JOINER})⸺”", r"\1—”", xhtml)
 	xhtml = regex.sub(fr"([^\s‘“—][a-z\.]{se.WORD_JOINER})⸺\s?", r"\1—", xhtml)
@@ -111,15 +121,21 @@ def typogrify(xhtml: str, smart_quotes: bool = True) -> str:
 	# Remove spaces after two-em-dashes that do not appear to be elision
 	xhtml = regex.sub(fr"(\p{{Letter}}{{2,}}{se.WORD_JOINER})⸺\s", r"\1—", xhtml)
 
+	# Dash before closing double quote in clauses with more than two words is almost always a typo
+	xhtml = regex.sub(r"(“[^<]+?\s[^<]+?)-”", fr"\1{se.WORD_JOINER}—”", xhtml)
+
 	# Replace Mr., Mrs., and other abbreviations, and include a non-breaking space
-	xhtml = regex.sub(r"\b(Mr|Mr?s|Drs?|Profs?|Lieut|Fr|Lt|Capt|Pvt|Esq|Mt|St|MM|Mmes?|Mlles?)\.?(</abbr>)?\s+", fr"\1.\2{se.NO_BREAK_SPACE}", xhtml)
+	xhtml = regex.sub(r"\b(Mr|Mr?s|Drs?|Profs?|Lieut|Fr|Lt|Capt|Pvt|Esq|Mt|St|MM|Mmes?|Mlles?|Hon|Mdlle)\.?(</abbr>)?\s+", fr"\1.\2{se.NO_BREAK_SPACE}", xhtml)
+
+	# Include a non-breaking space after Mon. We can't include it in the above regex because `Mon` may appear as running French language (i.e. `Mon Dieu`)
+	xhtml = regex.sub(r"\bMon\.(</abbr>)?\s+", fr"Mon.\1{se.NO_BREAK_SPACE}", xhtml)
 
 	# \P{} is the inverse of \p{}, so this regex matches any of the abbrs followed by any punctuation except a period. We also 'or' against a word joiner,
 	# in case Mr. is run up against an em dash.
 	xhtml = regex.sub(fr"\b(Mr|Mr?s|Drs?|Profs?|Lieut|Fr|Lt|Capt|Pvt|Esq|Mt|St|MM|Mmes?|Mlles?)\.?(</abbr>)?([^\P{{Punctuation}}\.]|{se.WORD_JOINER})", r"\1.\2\3", xhtml)
 
 	# We added an nbsp after St. above. But, sometimes a name can be abbreviated, like `Bob St. M.`. In this case we don't want an nbsp because <abbr> is already `white-space: nowrap;`.
-	xhtml = regex.sub(fr"""<abbr class="name">St\.{se.NO_BREAK_SPACE}([A-Z])""", r"""<abbr class="name">St\. \1""", xhtml)
+	xhtml = regex.sub(fr"""<abbr epub:type="(z3998:[^"\s]+?name)">St\.{se.NO_BREAK_SPACE}([A-Z])""", r"""<abbr epub:type="\1">St\. \2""", xhtml)
 
 	xhtml = regex.sub(r"\bNo\.\s+([0-9]+)", fr"No.{se.NO_BREAK_SPACE}\1", xhtml)
 	xhtml = regex.sub(r"<abbr>No\.</abbr>\s+", fr"<abbr>No.</abbr>{se.NO_BREAK_SPACE}", xhtml)
@@ -209,7 +225,6 @@ def typogrify(xhtml: str, smart_quotes: bool = True) -> str:
 	xhtml = regex.sub(r"([0-9])\s+([ap])\.m\.", fr"\1{se.NO_BREAK_SPACE}\2.m.", xhtml, flags=regex.IGNORECASE)
 	xhtml = regex.sub(r"([0-9])\s+<abbr([^>]*?)>([ap])\.m\.", fr"\1{se.NO_BREAK_SPACE}<abbr\2>\3.m.", xhtml, flags=regex.IGNORECASE)
 
-	xhtml = xhtml.replace("Ph.D", "PhD")
 	xhtml = regex.sub(r"P\.?\s*S\.", r"P.S.", xhtml)
 
 	# Fractions
@@ -247,8 +262,9 @@ def typogrify(xhtml: str, smart_quotes: bool = True) -> str:
 	# Remove word joiners if the em dash is preceded by a space
 	xhtml = regex.sub(fr"(\s+){se.WORD_JOINER}—", r"\1—", xhtml)
 
-	# Fix a common typo
+	# Fix some common error cases
 	xhtml = regex.sub(r"‘n’", "’n’", xhtml)
+	xhtml = regex.sub(r"</(a|abbr|b|i|span)>‘(s|d)\b", r"</\1>’\2", xhtml)
 
 	# Remove periods from O.K. (also, it is not an abbreviation)
 	xhtml = regex.sub(r"O\.K\.", r"OK", xhtml)
@@ -274,30 +290,7 @@ def typogrify(xhtml: str, smart_quotes: bool = True) -> str:
 
 	return xhtml
 
-def hyphenate_file(filename: Path, language: Optional[str], ignore_h_tags: bool = False) -> None:
-	"""
-	Add soft hyphens to an XHTML file.
-
-	INPUTS
-	filename: A filename containing well-formed XHTML.
-	language: An ISO language code, like en-US, or None to auto-detect based on XHTML input
-	ignore_h_tags: True to not hyphenate within <h1-6> tags
-
-	OUTPUTS
-	None.
-	"""
-
-	with open(filename, "r+", encoding="utf-8") as file:
-		xhtml = file.read()
-
-		processed_xhtml = se.typography.hyphenate(xhtml, language, ignore_h_tags)
-
-		if processed_xhtml != xhtml:
-			file.seek(0)
-			file.write(processed_xhtml)
-			file.truncate()
-
-def hyphenate(xhtml: str, language: Optional[str], ignore_h_tags: bool = False) -> str:
+def hyphenate(xhtml: Union[str, EasyXmlTree], language: Optional[str], ignore_h_tags: bool = False) -> str:
 	"""
 	Add soft hyphens to a string of XHTML.
 
@@ -310,7 +303,14 @@ def hyphenate(xhtml: str, language: Optional[str], ignore_h_tags: bool = False)
 	A string of XHTML with soft hyphens inserted in words. The output is not guaranteed to be pretty-printed.
 	"""
 
-	dom = EasyXhtmlTree(xhtml)
+	output_xhtml = ""
+
+	if isinstance(xhtml, EasyXmlTree):
+		dom = xhtml
+		output_xhtml = dom.to_string()
+	else:
+		dom = EasyXmlTree(xhtml)
+		output_xhtml  = xhtml
 
 	if language is None:
 		try:
@@ -337,8 +337,6 @@ def hyphenate(xhtml: str, language: Optional[str], ignore_h_tags: bool = False)
 	reading_tag_name = False
 	in_h_tag = False
 	pos = 1
-	h_opening_tag_pattern = regex.compile("^h[1-6]$")
-	h_closing_tag_pattern = regex.compile("^/h[1-6]$")
 
 	# The general idea here is to read the whole contents of the <body> tag character by character.
 	# If we hit a <, we ignore the contents until we hit the next >.
@@ -366,10 +364,10 @@ def hyphenate(xhtml: str, language: Optional[str], ignore_h_tags: bool = False)
 			process = True
 
 		# Do we ignore <h1-6> tags?
-		if not reading_tag_name and h_opening_tag_pattern.match(tag_name):
+		if not reading_tag_name and regex.match(r"^h[1-6]$", tag_name):
 			in_h_tag = True
 
-		if not reading_tag_name and h_closing_tag_pattern.match(tag_name):
+		if not reading_tag_name and regex.match(r"^/h[1-6]$", tag_name):
 			in_h_tag = False
 
 		if ignore_h_tags and in_h_tag:
@@ -384,9 +382,9 @@ def hyphenate(xhtml: str, language: Optional[str], ignore_h_tags: bool = False)
 
 		pos = pos + 1
 
-	xhtml = regex.sub(r"(<body[^>]+?>).+?<\/body>", fr"\1{result}</body>", xhtml, flags=regex.DOTALL)
+	output_xhtml = regex.sub(r"(<body[^>]*?>).+?<\/body>", fr"\1{result}</body>", output_xhtml, flags=regex.DOTALL)
 
-	return xhtml
+	return output_xhtml
 
 def guess_quoting_style(xhtml: str) -> str:
 	"""
@@ -408,11 +406,8 @@ def guess_quoting_style(xhtml: str) -> str:
 	# Quote style percentage above the threshold is returned.
 	threshold = 80
 
-	ldq_pattern = regex.compile(r"\t*<p>(.*?)“")
-	lsq_pattern = regex.compile(r"\t*<p>(.*?)‘")
-
-	lsq_count = len([m for m in lsq_pattern.findall(xhtml) if m.count("“") == 0])
-	ldq_count = len([m for m in ldq_pattern.findall(xhtml) if m.count("‘") == 0])
+	lsq_count = len([m for m in regex.findall(r"\t*<p>(.*?)‘", xhtml) if m.count("“") == 0])
+	ldq_count = len([m for m in regex.findall(r"\t*<p>(.*?)“", xhtml) if m.count("‘") == 0])
 
 	detected_style = "unsure"
 	american_percentage = 0
